/***********************************************************************//**
* @file		AC30M1x164_hal_frt.h
* @brief	Contains all macro definitions and function prototypes
* 			support for FRT firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


#ifndef _AC30M1x164_FRT_H_
#define _AC30M1x164_FRT_H_

/* Includes ----------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @brief FRT Flags
 */
#define	FRT_MATCH_FLAG		0x00			/*!< FRT Match Interrupt flag bit */	
#define	FRT_OVERFLOW_FLAG	0x01			/*!< FRT Overflow Interrupt flag bit */
#define	FRT_READACK_FLAG	0x02			/*!< FRT Read Counter Acknowledge bit */

/**
 * @brief FRT Clock Source
 */
#define	FRT_LSIOSC			0x00			/*!< Low Speed Internal Oscillator clock (40KHz) */
#define	FRT_EXTOSC			0x01			/*!< External Oscillator clock divided by 32 */
#define	FRT_SUBOSC			0x02			/*!< Sub Oscillator clock */


/* Public Types --------------------------------------------------------------- */
/***********************************************************************
 * @brief FRT enumeration
**********************************************************************/

/********************************************************************
* @brief FRT Mode Structure definition
**********************************************************************/

/********************************************************************
* @brief FRT Operation Structure definition
**********************************************************************/
/* Public Functions ----------------------------------------------------------- */
HAL_Status_Type  HAL_FRT_ModeInit(FRT_Type *FRTx);
HAL_Status_Type  HAL_FRT_ClockSource(FRT_Type *FRTx, uint8_t ClkSource);
uint8_t HAL_FRT_GetClockSource(FRT_Type *FRTx);
HAL_Status_Type  HAL_FRT_SetPeriod(FRT_Type *FRTx, uint32_t Period);
uint32_t HAL_FRT_GetPeriod(FRT_Type *FRTx);
uint32_t HAL_FRT_GetCounterValue(FRT_Type *FRTx);
HAL_Status_Type  HAL_FRT_Run(FRT_Type *FRTx, uint8_t isset);
HAL_Status_Type  HAL_FRT_MatchInterrupt(FRT_Type *FRTx, uint8_t isset);
HAL_Status_Type  HAL_FRT_OverflowInterrupt(FRT_Type *FRTx, uint8_t isset);
HAL_Status_Type  HAL_FRT_CounterMatchClearDisableBit(FRT_Type *FRTx, uint8_t isset);
HAL_Status_Type  HAL_FRT_SetStatus(FRT_Type *FRTx, uint8_t StatusType);
uint8_t HAL_FRT_GetStatus(FRT_Type *FRTx);
HAL_Status_Type  HAL_FRT_Counter(FRT_Type *FRTx, uint8_t action);
uint32_t HAL_FRT_GetCounterVal(FRT_Type *FRTx);
HAL_Status_Type  HAL_FRT_ClearCounter(FRT_Type *FRTx);
HAL_Status_Type HAL_FRT_HoldCounter(FRT_Type *FRTx, uint8_t isHold);

#ifdef __cplusplus
}
#endif


#endif /* _AC30M1x164_FRT_H_ */

/* --------------------------------- End Of File ------------------------------ */
