/**********************************************************************//**
* @file		AC30M1x164_hal_mpwm.h
* @brief	Contains all macro definitions and function prototypes
* 			support for MPWM firmware library on AC30M1x164
* @version	1.00
* @date		
* @author ABOV M team
*
* Copyright(C)  2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


#ifndef _AC30M1x164_MPWM_H_
#define _AC30M1x164_MPWM_H_

/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/* Public Functions ----------------------------------------------------------- */
/* UART Init/DeInit functions --------------------------------------------------*/
HAL_Status_Type  HAL_MPWM_Init(MPWM_Type *MPWMx);
HAL_Status_Type  HAL_MPWM_DeInit(MPWM_Type* MPWMx);

HAL_Status_Type  HAL_MPWM_Start(MPWM_Type* MPWMx, FunctionalState NewState);

HAL_Status_Type  HAL_MPWM_SetOutput(MPWM_Type* MPWMx, uint32_t Data);
HAL_Status_Type  HAL_MPWM_SetFOutput(MPWM_Type* MPWMx, uint32_t Data);

HAL_Status_Type  HAL_MPWM_SetProtCtrl(MPWM_Type* MPWMx, uint32_t Data, uint32_t ch);
HAL_Status_Type  HAL_MPWM_SetProtStat(MPWM_Type* MPWMx, uint32_t Data, uint32_t ch);
	
HAL_Status_Type  HAL_MPWM_ConfigInterrupt(MPWM_Type* MPWMx, uint32_t MPIntCfg, FunctionalState NewState);

HAL_Status_Type  HAL_MPWM_Cmd(MPWM_Type* MPWMx, uint32_t motorb, uint32_t update, uint32_t tup, uint32_t bup, uint32_t mchmode, uint32_t updown);

HAL_Status_Type  HAL_MPWM_SetPeriod(MPWM_Type* MPWMx, uint32_t period);
HAL_Status_Type  HAL_MPWM_SetUDuty(MPWM_Type* MPWMx, uint32_t udutyH, uint32_t udutyL);
HAL_Status_Type  HAL_MPWM_SetVDuty(MPWM_Type* MPWMx, uint32_t vdutyH, uint32_t vdutyL);
HAL_Status_Type  HAL_MPWM_SetWDuty(MPWM_Type* MPWMx, uint32_t wdutyH, uint32_t wdutyL);

HAL_Status_Type  HAL_MPWM_SetDeadTime(MPWM_Type* MPWMx, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t clkdata);

HAL_Status_Type  HAL_MPWM_SetATR(MPWM_Type* MPWMx, uint32_t atudt, uint32_t atmod, uint32_t atcnt, uint32_t ch);

#ifdef __cplusplus
}
#endif


#endif /* _AC30M1x164_MPWM_H_ */

/* --------------------------------- End Of File ------------------------------ */
