/**********************************************************************//**
* @file		AC30M1x64_hal_pcu.h
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library on AC30M1x64
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


#ifndef _AC30M1x164_PCU_H_
#define _AC30M1x164_PCU_H_

/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* Private macros ------------------------------------------------------------- */

#define PORT_ACCESS_EN()   do { PORTEN->EN=0x15; PORTEN->EN=0x51; } while(0)
#define PORT_ACCESS_DIS()  do { PORTEN->EN=0x00; } while(0) 

#define PCU_MR_FUNC_Msk                        (0x03UL)                            
#define PCU_CR_MODE_Msk                        (0x03UL)     

/* ================================================================================ */
/* ================          struct 'PCU' Position & Mask          ================ */
/* ================================================================================ */


/* Pin function */
#define FUNC0 			0x0				/** Function 0 	*/
#define FUNC1 			0x1				/** Function 1 	*/
#define FUNC2 			0x2				/** Function 2	*/
#define FUNC3 			0x3				/** Function 3	*/

//==========================================================================
// 	PAMR
//==========================================================================
#define PA0_MUX_PA0							(0)
#define PA0_MUX_T2IO						(1)
#define PA0_MUX_AN0							(3)

#define PA1_MUX_PA1							(0)
#define PA1_MUX_T3IO						(1)
#define PA1_MUX_AN1							(3)

#define PA2_MUX_PA2							(0)
#define PA2_MUX_SS0							(1)
#define PA2_MUX_WDTO						(2)
#define PA2_MUX_AN2							(3)

#define PA3_MUX_PA3							(0)
#define PA3_MUX_SCK0						(1)
#define PA3_MUX_STBO						(2)
#define PA3_MUX_AN3							(3)

#define PA4_MUX_PA4							(0)
#define PA4_MUX_AN4							(3)

#define PA5_MUX_PA5							(0)
#define PA5_MUX_AN5							(3)

#define PA6_MUX_PA6							(0)
#define PA6_MUX_T0IO						(1)
#define PA6_MUX_AN6							(3)

#define PA7_MUX_PA7							(0)
#define PA7_MUX_T1IO						(1)
#define PA7_MUX_AN7							(3)

#define PA8_MUX_PA8							(0)
#define PA8_MUX_T2IO						(1)
#define PA8_MUX_T0IO						(2)
#define PA8_MUX_AN8							(3)

#define PA9_MUX_PA9							(0)
#define PA9_MUX_T3IO						(1)
#define PA9_MUX_T1IO						(2)
#define PA9_MUX_AN9							(3)

#define PA10_MUX_PA10					(0)
#define PA10_MUX_AN10					(3)

#define PA11_MUX_PA11					(0)
#define PA11_MUX_AN11					(3)

#define PA12_MUX_PA12					(0)
#define PA12_MUX_T0IO					(1)

#define PA13_MUX_PA13					(0)
#define PA13_MUX_T1IO					(1)

#define PA14_MUX_PA14					(0)
#define PA14_MUX_T2IO					(1)

#define PA15_MUX_PA15					(0)
#define PA15_MUX_T3IO					(1)

//==========================================================================
// 	PBMR
//==========================================================================
#define PB0_MUX_PB0					(0)
#define PB0_MUX_MPWM0UH		(1)
#define PB0_MUX_SS0				(2)

#define PB1_MUX_PB1					(0)
#define PB1_MUX_MPWM0UL		(1)
#define PB1_MUX_SCK0				(2)

#define PB2_MUX_PB2					(0)
#define PB2_MUX_MPWM0VH		(1)				
#define PB2_MUX_MOSI0				(2)

#define PB3_MUX_PB3					(0)
#define PB3_MUX_MPWM0VL		(1)				
#define PB3_MUX_MISO0				(2)

#define PB4_MUX_PB4					(0)
#define PB4_MUX_MPWM0WH		(1)

#define PB5_MUX_PB5					(0)
#define PB5_MUX_MPWM0WL		(1)

#define PB6_MUX_PB6					(0)
#define PB6_MUX_PRTIN0			(1)

#define PB7_MUX_PB7					(0)
#define PB7_MUX_OVIN0 				(1)


//==========================================================================
// 	PCMR
//		
//
//==========================================================================
#define PC0_MUX_PC0							(0)
#define PC0_MUX_SWCLK					(1)
#define PC0_MUX_RXD1						(2)

#define PC1_MUX_PC1							(0)
#define PC1_MUX_SWDIO					(1)
#define PC1_MUX_TXD1						(2)

#define PC2_MUX_PC2							(0)

#define PC3_MUX_PC3							(0)

#define PC4_MUX_PC4							(0)
#define PC4_MUX_nTRST						(1)
#define PC4_MUX_T0IO						(2)

#define PC5_MUX_PC5							(0)
#define PC5_MUX_RXD1						(1)
#define PC5_MUX_T1IO							(2)

#define PC6_MUX_PC6							(0)
#define PC6_MUX_TXD1						(1)
#define PC6_MUX_T2IO						(2)

#define PC7_MUX_PC7							(0)
#define PC7_MUX_SCL0						(1)
#define PC7_MUX_T3IO						(2)

#define PC8_MUX_PC8							(0)
#define PC8_MUX_SDA0						(1)

#define PC9_MUX_PC9							(0)
#define PC9_MUX_CLKO						(1)

#define PC10_MUX_PC10						(0)
#define PC10_MUX_nRESET					(1)

#define PC11_MUX_PC11						(0)
#define PC11_MUX_T0IO 						(2)

#define PC12_MUX_PC12						(0)
#define PC12_MUX_T3IO 						(1)
#define PC12_MUX_XIN						(3)

#define PC13_MUX_PC13						(0)
#define PC13_MUX_T2IO 						(1)
#define PC13_MUX_XOUT						(3)

#define PC14_MUX_PC14						(0)
#define PC14_MUX_RXD0						(1)

#define PC15_MUX_PC15						(0)
#define PC15_MUX_TXD0						(1)

//==========================================================================
// 	PDMR
//==========================================================================
#define PD0_MUX_PD0							(0)
#define PD0_MUX_SS0							(1)				

#define PD1_MUX_PD1							(0)
#define PD1_MUX_SCK0						(1)			

#define PD2_MUX_PD2							(0)
#define PD2_MUX_MOSI0						(1)				
#define PD2_MUX_SCL0						(2)		
#define PD2_MUX_SXOUT					(3)		

#define PD3_MUX_PD3							(0)
#define PD3_MUX_MISO0						(1)				
#define PD3_MUX_SDA0						(2)
#define PD3_MUX_SXIN						(3)			

/*
 * @brief 	PCU port mode enumerate definition
 */
typedef enum {
	PUSH_PULL_OUTPUT = 0,
	OPEN_DRAIN_OUTPUT,
	INPUT,
	ANALOG
}PCU_PORT_MODE;

enum {
	IER_DISABLE = 0,
	IER_LEVEL,
	IER_EDGE = 3
};

enum {
	ICR_PROHIBIT_INT = 0,
	ICR_LOW_LEVEL_INT = 1,
	ICR_FALLING_EDGE_INT = ICR_LOW_LEVEL_INT,
	ICR_HIGH_LEVEL_INT =2,
	ICR_RISING_EDGE_INT = ICR_HIGH_LEVEL_INT,
	ICR_BOTH_EDGE_INT =3	
};


/* Public Functions ----------------------------------------------------------- */
void HAL_GPIO_ConfigFunction(PCU_Type *PCx, uint8_t pin_no, uint32_t func);
void HAL_GPIO_ConfigOutput(PCU_Type *PCx, uint8_t pin_no, PCU_PORT_MODE dir_type);
void HAL_GPIO_ConfigPullup (PCU_Type *PCx, uint8_t pin_no, FunctionalState pullup);

void HAL_GPIO_EXTI_Config(PCU_Type *PCx, uint8_t pin_no, uint8_t pin_en, uint8_t int_mode);
uint32_t HAL_GPIO_GetInterruptStatus(PCU_Type *PCx);
void HAL_GPIO_EXTI_ClearPin(PCU_Type *PCx, uint32_t value);

void HAL_GPIO_ConfigureDebounce (PCU_Type *PCx, uint8_t pin_no, FunctionalState debounce);

void HAL_GPIO_SetPin(GPIO_Type *Px, uint16_t bitValue);
void HAL_GPIO_ClearPin(GPIO_Type *Px, uint16_t bitValue);
uint16_t HAL_GPIO_ReadPin(GPIO_Type *Px);
void HAL_GPIO_WritePin(GPIO_Type *Px, uint16_t Value);

#ifdef __cplusplus
}
#endif

#endif /* end _AC30M1x164_PCU_H_ */

/* --------------------------------- End Of File ------------------------------ */
