/**********************************************************************//**
* @file		AC30M1x64_hal_scu.h
* @brief	Contains all macro definitions and function prototypes
* 			support for SCU firmware library on AC30M1x64
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


#ifndef _AC30M1x164_SCU_H_
#define _AC30M1x164_SCU_H_

/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* Private macros ------------------------------------------------------------- */
/* External sub crystal oscillator control */
#define EXSOSC_STOP		 			0x0				
#define EXSOSC_EN				 			0x2				
#define EXSOSC_EN_DIV2 			0x3		

/* External sub crystal oscillator stable timeout */
#define SOSC_STARTUP_TIMEOUT 0xFFFFFFFFUL

	/* Internal ring oscillator control */
#define INRINGOSC_STOP		 			0x0				
#define INRINGOSC_EN				 			0x2				
#define INRINGOSC_EN_DIV2 		0x3		

	/* Internal high speed oscillator control */
#define HSIOSC_STOP		 			0x0				
#define HSIOSC_EN				 			0x2				
#define HSIOSC_EN_DIV2 		0x3	

/* External crystal oscillator control */
#define EXOSC_STOP		 			0x0				
#define EXOSC_EN				 			0x2				
#define EXOSC_EN_DIV2 			0x3		

/* External crystal oscillator stable timeout */
#define MOSC_STARTUP_TIMEOUT 0xFFFFFFFFUL

/* system clock control setting */
#define SC_INRINGOSC	0x0
#define SC_EXSOSC	0x1
#define SC_HSIOSC          0x4
#define SC_EXOSC          0x6

enum {
	SYSTICK_TYPE=0,
	PWM0_TYPE,
	WDT_TYPE,
	TEXT_TYPE,
	PAD_TYPE,
	PBD_TYPE,
	PCD_TYPE,
	PDD_TYPE,	
	UARTC_TYPE,	
	ADCC_TYPE		
};

enum {
	CLKSRC_LSI=0,
	CLKSRC_MCLK=4,
	CLKSRC_HSI=5,	
	CLKSRC_EOSC=6,
	CLKSRC_SOSC=7	
};

enum {
	LVD_LEVEL_1_65 = 0,
	LVD_LEVEL_2_5,
	LVD_LEVEL_3_6,
	LVD_LEVEL_4_2
};

#define WAKEUP_GPIOD     (1UL<<11)
#define WAKEUP_GPIOC     (1UL<<10)
#define WAKEUP_GPIOB     (1UL<<9)
#define WAKEUP_GPIOA     (1UL<<8)
#define WAKEUP_FRT        (1UL<<2)
#define WAKEUP_WDT        (1UL<<1)
#define WAKEUP_LVD        (1UL<<0)


/* Public Functions ----------------------------------------------------------- */
void HAL_SCU_ClockInit(void);

void HAL_SCU_LSE_ClockConfig(uint8_t esosccon);
Status HAL_SCU_LSE_ClockMonitoring(void);

void HAL_SCU_LSI_ClockConfig(uint8_t ringosc); // in ring osc set 
void HAL_SCU_HSI_ClockConfig(uint8_t hsiosc); // hsi osc set

void HAL_SCU_HSE_ClockConfig(uint8_t eosccon);	 // exosc set
Status HAL_SCU_HSE_ClockMonitoring(void); // check for MOSC stable 


void HAL_SCU_SystemClockChange(uint8_t sysclkcon); // set system clock ==> final MCLK
void HAL_SCU_ClockOutput(uint8_t divval, FunctionalState endis);

void HAL_SCU_MiscClockConfig(uint8_t mccrnum, uint8_t type, uint8_t clksrc, uint8_t clkdiv);

void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState);
uint32_t HAL_SCU_GetWakeUpSRCStatus(void);
void SCU_ClearWakeUpSRC(uint32_t WakeUpStatus);

void HAL_SCU_LVR_Cmd(FunctionalState NewState);
	
uint32_t HAL_SCU_GetNMIStatus(void);
void HAL_SCU_SetNMI(uint32_t nmisrc);

	
#ifdef __cplusplus
}
#endif

#endif /* end _AC30M1x164_SCU_H_ */

/* --------------------------------- End Of File ------------------------------ */
