/**********************************************************************//**
* @file		AC30M1x164_hal_wdt.h
* @brief	Contains all macro definitions and function prototypes
* 			support for WDT firmware library on AC30M1x164
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Peripheral group ----------------------------------------------------------- */
#ifndef _AC30M1x164_WDT_H_
#define _AC30M1x164_WDT_H_

/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* --------------------- BIT DEFINITIONS -------------------------------------- */
/** WDT interrupt enable bit */
#define WDT_CON_CKSEL			    ((uint16_t)(1<<3))  // wdt clock sel	
#define WDT_CON_WDTEN			    ((uint16_t)(1<<4))  // wdt enable
#define WDT_CON_WDTRE			    ((uint16_t)(1<<6))  // wdt reset enable 
#define WDT_CON_WDTIE			    ((uint16_t)(1<<7))  // wdt timer count interrupt enable 
#define WDT_CON_WUF				    ((uint16_t)(1<<8))  // wdt underflow flag 	
#define WDT_CON_WDBG			    ((uint16_t)(1<<15))  // wdt dbg 		
	/* Public Types --------------------------------------------------------------- */
/**********************************************************************
 * @brief WDT structure definitions
 **********************************************************************/
enum {
		WDT_DIV_1=0,
		WDT_DIV_4,
		WDT_DIV_8,
		WDT_DIV_16,
		WDT_DIV_32,
		WDT_DIV_64,
		WDT_DIV_128,
		WDT_DIV_256	
};

typedef struct Wdt_Config
{
	uint8_t wdtResetEn;			/**< if ENABLE -> the Reset bit is enabled				*/
	uint8_t wdtCountEn;			/**< if ENABLE -> the Protect bit is enabled			*/
	uint8_t wdtClkSel;			/* SET -> external, RESET -> PCLK */
	uint16_t wdtPrescaler;		/* prescaler */
	
	uint32_t wdtTmrConst;		/**< Set the constant value to timeout the WDT (us)		*/
} WDT_CFG_Type;

/* Public Functions ----------------------------------------------------------- */
HAL_Status_Type  HAL_WDT_Init(void);
HAL_Status_Type  HAL_WDT_ReloadTimeCounter(uint32_t TimeOut);
HAL_Status_Type  HAL_WDT_Configure(WDT_CFG_Type wdtCfg);
HAL_Status_Type  HAL_WDT_Start(FunctionalState ctrl);
uint32_t HAL_WDT_GetCurrentCount(void);

#ifdef __cplusplus
}
#endif

#endif /* _AC30M1x164_WDT_H_ */

/* --------------------------------- End Of File ------------------------------ */
