/**********************************************************************//**
* @file		AC30M1x164_hal_wdt.c
* @brief	Contains all functions support for WDT firmware library
* 			on AC30M1x164
* @version	1.00
* @date		
* @authorABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser  products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/


/* Peripheral group ----------------------------------------------------------- */
/* Includes ------------------------------------------------------------------- */
#include "AC30M1x64_hal_wdt.h"
#include "AC30M1x64_hal_libcfg.h"



uint32_t _WDTCLK;

/* Public Functions ----------------------------------------------------------- */
/**********************************************************************//**
* @brief 		Initial for Watchdog function
* @param[in]	none
* @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_WDT_Init(void)
{
	WDT->CON = 0; 
	WDT->LR = 0;
	return HAL_OK;
}

/*********************************************************************//**
 * @brief 		Update WDT timeout value and feed
 * @param[in]	TimeOut	TimeOut value to be updated, should be in range:
 * 				2048 .. 134217728
 * @return		None
 *********************************************************************/
HAL_Status_Type  HAL_WDT_ReloadTimeCounter(uint32_t TimeOut)
{
	WDT->LR = TimeOut;
	return HAL_OK;
}

/**********************************************************************//**
* @brief 		Enable/Disable WWDT activity
* @param[in]	None
* @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_WDT_Configure(WDT_CFG_Type wdtCfg)
{
	uint16_t reg_val;
	
	reg_val = 0;
	
	WDT->LR = wdtCfg.wdtTmrConst;
	
	reg_val = wdtCfg.wdtPrescaler;
	
	if(wdtCfg.wdtResetEn == ENABLE) {
		reg_val |= WDT_CON_WDTRE;
	}
	else {
		reg_val &= ~WDT_CON_WDTRE;
	}

	if(wdtCfg.wdtCountEn) {
		reg_val |= WDT_CON_WDTIE;
	}
	else {
		reg_val &= ~WDT_CON_WDTIE;
	}

	if(wdtCfg.wdtClkSel) {
		reg_val |= WDT_CON_CKSEL; //external clock from MCCR3
	}
	else {
		reg_val &= ~WDT_CON_CKSEL; // PCLK
	}
	
//	WDT->CON = reg_val | (WDT_CON_WDBG);	
	WDT->CON = reg_val;
	return HAL_OK;

}

/**********************************************************************//**
* @brief 		Enable WWDT activity
* @param[in]	FunctionalState ctrl 
*						- DISABLE: wdt enable
*						- ENABLE: wdt disable 
* @return 		None
 **********************************************************************/
HAL_Status_Type  HAL_WDT_Start(FunctionalState ctrl)
{
	if (ctrl == ENABLE){
		WDT->CON |= WDT_CON_WDTEN;
	}
	else {
		WDT->CON &= ~WDT_CON_WDTEN;		
	}
	return HAL_OK;
}

/*********************************************************************//**
 * @brief 		Get the current value of WDT
 * @param[in]	None
 * @return		current value of WDT
 *********************************************************************/
uint32_t HAL_WDT_GetCurrentCount(void)
{
	return WDT->CNT;
}



/* --------------------------------- End Of File ------------------------------ */
