/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void TIMER0_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void delay(void);
void GPIO_Configure(void);
void ADC_Configure(void);
void ADC_PollingRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" ADC demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using ADC in polling mode \n\r"
"************************************************\n\r";
uint16_t adcval[8];
uint32_t i;
uint32_t adcavg;
uint32_t tmp;
ADC_CFG_Type AD_config;


/**********************************************************************//**
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************//**
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 1000000 ; i++) ;
}

/*********************************************************************//**
 * @brief		adcaverage
 * @param[in]	None
 * @return 		None
 **********************************************************************/
uint16_t adcaverage(uint16_t *val)
{
	uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;			
	for (i=0;i<8;i++){	
		adcavg+=(val[i]>>4);
	}
	adcavg>>=3;
	
	return 	(adcavg);
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PCA, 0, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 0, PA0_MUX_AN0);
	
	HAL_GPIO_ConfigOutput(PCA, 1, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 1, PA1_MUX_AN1);;
	
	HAL_GPIO_ConfigOutput(PCA, 2, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 2, PA2_MUX_AN2);
	
	HAL_GPIO_ConfigOutput(PCA, 3, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 3, PA3_MUX_AN3);
}



/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	AD_config.Mode = ADC_SINGLE_MODE;
	AD_config.SampingTime=0x1f;
	AD_config.SeqCnt = 1;
	AD_config.RestartEn = 0;
	AD_config.TrgSel = ADC_TRIGGER_DISABLE;
	
	AD_config.UseClk = ADC_INTERNAL_CLK;
	AD_config.InClkDiv = 0;//0:system clock, 1:stop, n: (system clock)/n 
	if(HAL_ADC_Init(AD, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
    		Error_Handler();
	}
}
	

/**********************************************************************
 * @brief		ADC_InterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_PollingRun(void)
{	
	
		HAL_ADC_ChannelSel(AD, 0); // select ch
		
		for (i=0;i<(8);i++)
		{
			HAL_ADC_ClearStatus(AD,ADC_STAT_SINGLE);
			HAL_ADC_Start(AD); // start 	
			while((HAL_ADC_GetStatus(AD) & (ADC_STAT_SINGLE)) !=(ADC_STAT_SINGLE)){}				
			adcval[i]=HAL_ADC_GetData(AD);
		}
		
		adcavg=adcaverage(adcval);
		#ifdef _DEBUG_MSG
		_DBG("AN0 = "); _DBH16(adcavg); _DBG(" ");
		#endif

	
		HAL_ADC_ChannelSel(AD, 1);	
		for (i=0;i<(8);i++)
		{
			HAL_ADC_ClearStatus(AD,ADC_STAT_SINGLE);
			HAL_ADC_Start(AD); // start 	
			while((HAL_ADC_GetStatus(AD) & (ADC_STAT_SINGLE)) !=(ADC_STAT_SINGLE)){}				
			adcval[i]=HAL_ADC_GetData(AD);
		}
		
		adcavg=adcaverage(adcval);
		#ifdef _DEBUG_MSG
		_DBG("AN1 = "); _DBH16(adcavg); _DBG(" ");
		#endif

		
		HAL_ADC_ChannelSel(AD, 2);			
		for (i=0;i<(8);i++)
		{
			HAL_ADC_ClearStatus(AD,ADC_STAT_SINGLE);
			HAL_ADC_Start(AD); // start 	
			while((HAL_ADC_GetStatus(AD) & (ADC_STAT_SINGLE)) !=(ADC_STAT_SINGLE)){}				
			adcval[i]=HAL_ADC_GetData(AD);
		}
		
		adcavg=adcaverage(adcval);
		#ifdef _DEBUG_MSG
		_DBG("AN2 = "); _DBH16(adcavg); _DBG(" ");
		#endif

		HAL_ADC_ChannelSel(AD, 3); // select ch
		for (i=0;i<(8);i++)
		{
			HAL_ADC_ClearStatus(AD,ADC_STAT_SINGLE);
			HAL_ADC_Start(AD); // start 	
			while((HAL_ADC_GetStatus(AD) & (ADC_STAT_SINGLE)) !=(ADC_STAT_SINGLE)){}				
			adcval[i]=HAL_ADC_GetData(AD);
		}
		
		adcavg=adcaverage(adcval);
		#ifdef _DEBUG_MSG
		_DBG("AN3 = "); _DBH16(adcavg); _DBG_(" ");
		#endif

		//for(tmp = 0; tmp < 1000000; tmp++);
		delay();

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*ADC Configure*/
	ADC_Configure();
  
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
  		 /* Infinite loop */
		ADC_PollingRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
