/***********************************************************************
* @file		AC30M1x64_SystemClock.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
***********************************************************************/
/***********************************************************************
 *                          AC30M1x64 Device 
 *------------------------------------------------------------------------
 *    System Clock source       | High Speend Internal oscillator (HSI)
 *------------------------------------------------------------------------
 *    SYSCLK(MHz)               | 40MHz
 *------------------------------------------------------------------------
 *    HCLK(MHz) - Core Clock    | 40MHz
 *------------------------------------------------------------------------
  *   PCLK(MHz) - Peri Clock    | 40MHz
 *------------------------------------------------------------------------
*************************************************************************/
#include "main_conf.h"

/* Private typedef ------------------------------------------------------*/
/* Private define -------------------------------------------------------*/
/* Private macro --------------------------------------------------------*/
/* Private variables ----------------------------------------------------*/
/* Private define -------------------------------------------------------*/


//#define USED_CLOCKSRC     SC_INRINGOSC    /* SystemClock LSI 			: 40KHz   */
//#define USED_CLOCKSRC     SC_EXSOSC       /* SystemClock  HSE 		: 8KHz     */
#define USED_CLOCKSRC     SC_HSIOSC         /* SystemClock  HSI 		: 40MHz     */
//#define USED_CLOCKSRC     SC_EXOSC        /* SystemClock LSE 			: 32.768KHz  */

/* System Clock source */ 		
#define HSI_OSC (40000000)    

/* Clock Out Selection ( Monitoring )*/
#define USED_CLKO


/* Private function prototypes ------------------------------------------*/
void SystemClock_Config (void);
/* Private variables ----------------------------------------------------*/


/**************************************************************************
 * @brief			Initialize default clock for A34M41x Board
 * @param[in]		None
 * @return			None
 **************************************************************************/
void SystemClock_Config (void)
{
	uint32_t i;

		/*!! For clock change, enable all clock source. !!*/
		SCU->CSCR = 0xAA;
		
		/* CLKO function setting. check PORT setting (PC9).*/
	#ifdef USED_CLKO
		HAL_SCU_ClockOutput(4,ENABLE);
	#endif
		SCU->CMR&=~(1<<7); //mclk monitoring disable

	/* 	SystemClock LSE 			: 32.768KHz */
	#if (USED_CLOCKSRC==SC_INRINGOSC) 

		HAL_SCU_LSE_ClockConfig(INRINGOSC_EN);
		for (i=0;i<100;i++);
		
		HAL_SCU_SystemClockChange(USED_CLOCKSRC);
		
		SystemCoreClock=40000;
		SystemPeriClock=40000;	

	/*SystemClock LSE 			: 32.768KHz  */
	#elif (USED_CLOCKSRC==SC_EXSOSC) //32.768kHz
		/* Before using this routine, you must set PORTs as SXIN and SXOUT.*/
		/* external SOSC enable 	*/
		HAL_SCU_LSE_ClockConfig(EXOSC_EN); 
		for (i=0;i<100;i++);
		
		/* wait for SOSC stable	*/
		HAL_SCU_LSE_ClockMonitoring();
		HAL_SCU_SystemClockChange(USED_CLOCKSRC);
		
		SystemCoreClock=32768; /*32768Hz*/
		SystemPeriClock=32768; /*32768Hz*/
		
	/*SystemClock  HSI 		: 40MHz  */	
	#elif (USED_CLOCKSRC==SC_HSIOSC) // 40Mhz
		HAL_SCU_HSI_ClockConfig(HSIOSC_EN); 
		for (i=0;i<100;i++);
		
		HAL_SCU_SystemClockChange(USED_CLOCKSRC);
		
		SystemCoreClock = HSI_OSC; //40Mhz  <- HSIOSC_EN
		SystemPeriClock = HSI_OSC; //40Mhz	

	/*SystemClock  LSE 		: 8MHz  */	
	#elif (USED_CLOCKSRC==SC_EXOSC) //4Mhz~16Mhz
		/* Before using this routine, you must set PORTs as XIN and XOUT.*/
		/* external MOSC enable 	*/
		HAL_SCU_HSE_ClockConfig(EXOSC_EN); 
		
		for (i=0;i<10;i++);

		/* wait for MOSC stable	*/
		HAL_SCU_HSE_ClockMonitoring();
		
		HAL_SCU_SystemClockChange(USED_CLOCKSRC);


		SystemCoreClock=16000000; //xMhz
		SystemPeriClock=16000000; //xMhz	
	#else 
		while(1) {}	
	 
	#endif

		SCU->CMR|=(1<<7); //mclk monitoring enable

		/* flash memory controller */
		FM->MR = 0x81;       // after changing 0x81 -> 0x28 in FMMR reg, flash access timing will be able to be set.
		FM->MR = 0x28;       // enter flash access timing changing mode
		/*	FM->CFG = (0x7858<<16) | (0<<8); */ 
		/*	FM->CFG = (0x7858<<16) | (1<<8);  */
		FM->CFG = (0x7858<<16) | (2<<8);  //flash access cycles 			
		/*	FM->CFG = (0x7858<<16) | (3<<8);  flash access cycles */			
																	// flash access time cannot overflow 20Mhz.
																	// ex) if MCLK=40Mhz, 
																	//       40/1 = 40 (can't set no wait)
																	//       40/2 = 20 (1 wait is ok)
																	// so, 1 wait is possible.
		FM->MR = 0;	      // exit flash access timing --> normal mode			

			
}



