/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void TIMER0_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void delay(void);
void GPIO_Configure(void);
void TIME_Configure(void);
void ADC_Configure(void);
void ADC_SeqRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" ADC demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using ADC in sequence mode \n\r"
"************************************************\n\r";
uint32_t fflag;

uint32_t adcirq_seq;
uint32_t adcirq_trigger;
TIMER_PERIODICCFG_Type T0_Config;
uint32_t count;
ADC_CFG_Type AD_config;
/**********************************************************************
 * @brief		ADC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(AD);

	if (status & ADC_STAT_SEQ){
		fflag=1;
		HAL_ADC_ClearStatus(AD, ADC_STAT_SEQ);	
		adcirq_seq++;
	}
	if (status & ADC_STAT_TRIGGER){
		HAL_ADC_ClearStatus(AD, ADC_STAT_TRIGGER);	
		adcirq_trigger++;
	}	
}

/**********************************************************************
 * @brief		TIMER0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((TIMER_GetStatus(T0) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(T0, TIMER_SR_MFB);
		count++;
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 1000000 ; i++) ;
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PCA, 0, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 0, PA0_MUX_AN0);
	
	HAL_GPIO_ConfigOutput(PCA, 1, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 1, PA1_MUX_AN1);;
	
	HAL_GPIO_ConfigOutput(PCA, 2, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 2, PA2_MUX_AN2);
	
	HAL_GPIO_ConfigOutput(PCA, 3, ANALOG);
	HAL_GPIO_ConfigFunction(PCA, 3, PA3_MUX_AN3);
}


/**********************************************************************
 * @brief		TIME_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{

	T0_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	T0_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	T0_Config.GRA = (500); 	// adc trigger signal 
	T0_Config.GRB = (10000); // 10msec			
	T0_Config.Cnt=0;		
	T0_Config.StartLevel=START_LOW;
	T0_Config.AdcTrgEn=ENABLE;
	
	if(HAL_TIMER_Init(T0, PERIODIC_MODE, &T0_Config) != HAL_OK)  
	{
		/* Initialization Error */
    		Error_Handler();
	}
	HAL_TIMER_ConfigInterrupt(T0, TIMER_INTCFG_MBIE, ENABLE);

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER0_IRQn, ((0x01<<1)|0x01));
  /* Enable Interrupt for TIMER0 channel */
	NVIC_EnableIRQ(TIMER0_IRQn);

}
/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	AD_config.Mode = ADC_SINGLE_MODE;
	AD_config.SampingTime=0x1f;
	AD_config.SeqCnt = 4;
	AD_config.RestartEn = 0;
	AD_config.TrgSel = ADC_TRIGGER_TIMER;
	
	AD_config.UseClk = ADC_INTERNAL_CLK;
	AD_config.InClkDiv = 2; // 8Mhz 
	if(HAL_ADC_Init(AD, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
   		 Error_Handler();
	}
	
	HAL_ADC_TriggerlSel(AD, 0x00000000); //trigger timer0
	HAL_ADC_ChannelSel(AD, 0x76543210);	//ch0, ch1, ch2, ch3
	
	HAL_ADC_ClearStatus(AD, 0xff); // clear status
	HAL_ADC_ConfigInterrupt(AD, ( ADC_INTEN_SEQ | ADC_INTEN_TRIGGER), ENABLE);
	
	NVIC_SetPriority(ADC_IRQn, 3);
	/* Enable Interrupt for ADC channel */
  	NVIC_EnableIRQ(ADC_IRQn);

}
	

/**********************************************************************
 * @brief		TIMER_Run
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Run(void)
{
	adcirq_seq = 0;
	adcirq_trigger =0;

	count=0;
	HAL_TIMER_Cmd(T0, ENABLE); // timer start	
}
/**********************************************************************
 * @brief		ADC_InterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_SeqRun(void)
{	
    if (AD_config.RestartEn==0) 
	{
			HAL_ADC_Start(AD); // start 	
	}
		
	fflag=0;
	while(fflag==0)
	{
	}
     #ifdef _DEBUG_MSG
		_DBG("AN0 = "); _DBH16(AD->DR0>>4); _DBG(" ");
		_DBG("AN1 = "); _DBH16(AD->DR1>>4); _DBG(" ");
		_DBG("AN2 = "); _DBH16(AD->DR2>>4); _DBG(" ");
		_DBG("AN3 = "); _DBH16(AD->DR3>>4); _DBG_(" ");			
	#endif
	//for(tmp = 0; tmp < 1000000; tmp++);
	delay();

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure TIMER peripheral*/
	TIMER_Configure();
	
	/*ADC Configure*/
	ADC_Configure();
  
	/*TIMER Start*/
	TIMER_Run();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
  		 /* Infinite loop */
		ADC_SeqRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
