/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define LEDCNT 10
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void delay(void);
void CLOCK_Configure(void);
void CLOCK_ChangeRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" Clock change Test \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"************************************************\n\r";




/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x1000>>2 ; i++) ;
}

/*********************************************************************//**
 * @brief		delay1
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay1(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x10000 ; i++) ;
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
	
	//Pin setting PB1
	HAL_GPIO_ConfigFunction(PCB, 1, PB1_MUX_PB1);
	HAL_GPIO_ConfigOutput(PCB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 1, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(1));
}



/**********************************************************************
 * @brief		CLOCK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CLOCK_Configure(void)
{
	// all clock enable for using system 
	HAL_SCU_LSE_ClockConfig(INRINGOSC_EN); 

	HAL_SCU_HSE_ClockConfig(EXOSC_EN); 
	HAL_SCU_HSE_ClockMonitoring();

	HAL_SCU_LSE_ClockConfig(EXSOSC_EN); 
	HAL_SCU_LSE_ClockMonitoring();

}
	


/**********************************************************************
 * @brief		Clock_ChangeRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CLOCK_ChangeRun(void)
{		
	char cmd;
	uint32_t i,j;
	uint32_t InFlag;

	
	
		if (InFlag==0)
		{
			_DBG("\n\r");
			cmd='s';			
      
			switch(cmd)
			{
				case 's':				
					j=0;
					while(1){
						if (j==100) break;
						
						HAL_SCU_SystemClockChange(SC_INRINGOSC);
						for (i=0;i<LEDCNT;i++){
							if (i%2){
								HAL_GPIO_SetPin(PB, _BIT(0));
								delay();
							}
							else {
								HAL_GPIO_ClearPin(PB, _BIT(0));
								delay();
							}
						}
						delay();	
						
						HAL_SCU_SystemClockChange(SC_EXSOSC);
						for (i=0;i<LEDCNT;i++){
							if (i%2){
								HAL_GPIO_SetPin(PB, _BIT(1));
								delay();
							}
							else {
								HAL_GPIO_ClearPin(PB, _BIT(1));
								delay();
							}
						}
						delay();			

						HAL_SCU_SystemClockChange(SC_EXOSC);						
						for (i=0;i<LEDCNT;i++){
							if (i%2){
								HAL_GPIO_SetPin(PB, _BIT(1));
								delay1();
								delay1();
							}
							else {
								HAL_GPIO_ClearPin(PB, _BIT(1));
								delay1();
								delay1();
							}
						}
						delay();

						HAL_SCU_SystemClockChange(SC_EXSOSC);
						for (i=0;i<LEDCNT;i++){
							if (i%2){
								HAL_GPIO_SetPin(PB, _BIT(1));
								delay();
							}
							else {
								HAL_GPIO_ClearPin(PB, _BIT(1));
								delay();
							}
						}
						delay();	
						
						HAL_SCU_SystemClockChange(SC_HSIOSC);							
						for (i=0;i<LEDCNT;i++){
							if (i%2){
								HAL_GPIO_SetPin(PB, _BIT(0));
								delay1();
								delay1();
							}
							else {
								HAL_GPIO_ClearPin(PB, _BIT(0));
								delay1();
								delay1();
							}
						}
						delay();	
						
						j++;
						#ifdef _DEBUG_MSG
						_DBG("loop count : "); _DBD32(j);
						_DBG("\n\r");
						#endif
						delay1();
					}
					break;	
			}		
		}		
		else 
		{
			#ifdef _DEBUG_MSG
			_DBG("\n\r");
			#endif
			InFlag=0;	
		}
}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();

	
	/*Clock Configure*/
	CLOCK_Configure();
  

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   		/* Infinite loop */
		CLOCK_ChangeRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
