/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define STARTMODE 1   // 0: BREG trigger 
                     			// 1: START bit trigger 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void GPIO_Configure(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"DIV64 demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"This example used to test DIV64 function\n\r"
"************************************************\n\r";
unsigned long long div_q;
unsigned long div_r;

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 1000000 ; i++) ;
}

/**********************************************************************
 * @brief		t_div
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void t_div(void)
{
	unsigned long long a;
	unsigned long b;
			
	_DBG("Divide test\n\r");	
	_DBG("A(unsigned 64bit) / B(unsigned 32bit)\n\r");	
	_DBG("> A :27000\n\r");
	a=27000;
	_DBG("> B :43\n\r");	
	b=43;

	_DBG("> A=0x");_DBH32(a>>32);_DBH32(a);
	_DBG("\n\r");

	_DBG("> B=0x");_DBH32(b);
	_DBG("\n\r");
	
	while(DIV64->CR & (1<<9)); //check busy
	
	DIV64->AREGH = (a>>32)&0xffffffff;
	DIV64->AREGL = (a)&0xffffffff;
	DIV64->BREG = (b);

#if STARTMODE	
	DIV64->CR = (1<<0); // start 	
#endif

	while(DIV64->CR & (1<<9)); //check busy	
	
	div_q =DIV64->QREGH;
	div_q<<=32;
	div_q |= DIV64->QREGL;
	div_r = DIV64->RREG;

	_DBG("> Result q=0x");_DBH32(div_q>>32);_DBH32(div_q);
	_DBG("\n\r");

	_DBG("> Result r=0x");	_DBH32(div_r);
	_DBG("\n\r");
			
	_DBG("Divide test end\n\r");		
}

/**********************************************************************
 * @brief		t_div2
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void t_div2(void)
{
	unsigned long long a;
	unsigned long b;
			
	_DBG("Divide test\n\r");	
	_DBG("A(unsigned 64bit) / B(unsigned 32bit)\n\r");	
	_DBG("> A :0x000000000000ffff\n\r");
	a=0x000000000000ffff;
//	a=2;	
	_DBG("> B :0xffffffff\n\r");	
	b=0xffffffff;
//	b=5;
	_DBG("> A=0x");_DBH32(a>>32);_DBH32(a);
	_DBG("\n\r");

	_DBG("> B=0x");_DBH32(b);
	_DBG("\n\r");
	
	while(DIV64->CR & (1<<9)); //check busy
	
	DIV64->AREGH = (a>>32)&0xffffffff;
	DIV64->AREGL = (a)&0xffffffff;
	DIV64->BREG = (b);
	
#if STARTMODE	
	DIV64->CR = (1<<0); // start 	
#endif

	while(DIV64->CR & (1<<9)); //check busy	
	
	div_q =DIV64->QREGH;
	div_q<<=32;
	div_q |= DIV64->QREGL;
	div_r = DIV64->RREG;

	_DBG("> Result q=0x");_DBH32(div_q>>32);_DBH32(div_q);
	_DBG("\n\r");

	_DBG("> Result r=0x");	_DBH32(div_r);
	_DBG("\n\r");
			
	_DBG("Divide test end\n\r");		
}

/**********************************************************************
 * @brief		t_div3
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void t_div3(void)
{
	unsigned long long a;
	unsigned long b;
			
	_DBG("Divide test\n\r");	
	_DBG("A(unsigned 64bit) / B(unsigned 32bit)\n\r");	
	_DBG("> A :0xffffffffffffffff\n\r");
	a=0xffffffffffffffff;
	_DBG("> B :0\n\r");	
	b=0;

	_DBG("> A=0x");_DBH32(a>>32);_DBH32(a);
	_DBG("\n\r");

	_DBG("> B=0x");_DBH32(b);
	_DBG("\n\r");
	
	while(DIV64->CR & (1<<9)); //check busy
	
	DIV64->AREGH = (a>>32)&0xffffffff;
	DIV64->AREGL = (a)&0xffffffff;
	DIV64->BREG = (b);
	
#if STARTMODE	
	DIV64->CR = (1<<0); // start 	
#endif

	while(DIV64->CR & (1<<9)); //check busy	
	
	div_q =DIV64->QREGH;
	div_q<<=32;
	div_q |= DIV64->QREGL;
	div_r = DIV64->RREG;

	_DBG("> Result q=0x");_DBH32(div_q>>32);_DBH32(div_q);
	_DBG("\n\r");

	_DBG("> Result r=0x");	_DBH32(div_r);
	_DBG("\n\r");
			
	_DBG("Divide test end\n\r");		
}

/**********************************************************************
 * @brief		t_div4
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void t_div4(void)
{
	unsigned long long a;
	unsigned long b;
			
	_DBG("Divide test\n\r");	
	_DBG("A(unsigned 64bit) / B(unsigned 32bit)\n\r");	
	_DBG("> A :0x0000ffffffffffff\n\r");
	a=0x0000ffffffffffff;
	_DBG("> B :0xffff\n\r");	
	b=0xffff;

	_DBG("> A=0x");_DBH32(a>>32);_DBH32(a);
	_DBG("\n\r");

	_DBG("> B=0x");_DBH32(b);
	_DBG("\n\r");
	
	while(DIV64->CR & (1<<9)); //check busy
	
	DIV64->AREGH = (a>>32)&0xffffffff;
	DIV64->AREGL = (a)&0xffffffff;
	DIV64->BREG = (b);

#if STARTMODE	
	DIV64->CR = (1<<0); // start 	
#endif

	while(DIV64->CR & (1<<9)); //check busy	
	
	div_q =DIV64->QREGH;
	div_q<<=32;
	div_q |= DIV64->QREGL;
	div_r = DIV64->RREG;

	_DBG("> Result q=0x");_DBH32(div_q>>32);_DBH32(div_q);
	_DBG("\n\r");

	_DBG("> Result r=0x");	_DBH32(div_r);
	_DBG("\n\r");
			
	_DBG("Divide test end\n\r");		
}
/**********************************************************************
 * @brief		t_cal
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void t_cal(void)
{
	char cmd;

//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<5)   // DIV64
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<5)   // DIV64
		; 

#if STARTMODE	
	DIV64->CR = (0<<4); // start operation mode 	
#else 
	DIV64->CR = (1<<4); // start operation mode 
	                                // 0:  START bit write operation will trigger the divide operation
	                                // 1:	 BREG register write operation will trigger the divide operation
#endif 
	
	while(1){
		_DBG("\n\rDIV64 test Program\n\r");
		_DBG("1. Divide test1\n\r");
		_DBG("2. Divide test2\n\r");
		_DBG("3. Divide test3\n\r");
		_DBG("4. Divide test4\n\r");
		
		_DBG("*");				
		getstring();
		if (InFlag){
			_DBG("\n\r");
			cmd=InData[0];
				
			switch(cmd){
				case '1': 
					t_div();					
					break;	
				case '2': 
					t_div2();					
					break;	
				case '3': 
					t_div3();					
					break;		
				case '4': 
					t_div4();					
					break;				
			}
		}
		else {
			_DBG("\n\r");
			InFlag=0;
			InCount=0;
		}
	}
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PD0
	HAL_GPIO_ConfigFunction(PCD, 0, PD0_MUX_PD0);
	HAL_GPIO_ConfigOutput(PCD, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 0, DISABLE);	
	HAL_GPIO_SetPin(PD, _BIT(0));
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();

	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
  		 /* Infinite loop */
		t_cal();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
