/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void GPIO_Configure(void);
void SysTick_Configure(void);
void FMC_PgmEraseRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" flash demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using flash_iap_pgm \n\r"
"************************************************\n\r";

uint32_t msec;
uint32_t t500msec;
uint32_t tflag;

/**********************************************************************
 * @brief		SysTick_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
	
	t500msec++;
	if (t500msec>=500){
		if (tflag==0){
			tflag=1;
			HAL_GPIO_SetPin(PB,_BIT(1));
		}
		else {
			tflag=0;
			HAL_GPIO_ClearPin(PB,_BIT(1));
		}
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/*********************************************************************//**
 * @brief		tt_flash_iap_pgm
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_pgm(void)
{
	int	i;
	unsigned char	fbuffer[128];
	
	for(i=0;i<128;i++) {
		fbuffer[i]= i;
	}
	__disable_irq();
	flash_write_sector(0,0xFF00,(unsigned long*)fbuffer);
	__enable_irq();
}

/*********************************************************************//**
 * @brief		tt_flash_iap_erase
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_erase(void)
{
	__disable_irq();
	flash_erase_sector(0,0xFF00);
	__enable_irq();
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
	
	// Test Pin setting PB1
	HAL_GPIO_ConfigFunction(PCB, 1, PB1_MUX_PB1);
	HAL_GPIO_ConfigOutput(PCB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 1, DISABLE);
	HAL_GPIO_SetPin(PB, _BIT(1));
}


/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	t500msec=0;tflag=0;
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 

}

	
/**********************************************************************
 * @brief		FMC_PgmEraseRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FMC_PgmEraseRun(void)
{

		tt_flash_iap_erase();
		__NOP();
	
		tt_flash_iap_pgm();
		__NOP();
		
			
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure SysTick peripheral*/
	SysTick_Configure();
  
	/*FMC Pgr, erase */
	FMC_PgmEraseRun();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
  	/* Infinite loop */
		msec = 500;
		while(msec);
		HAL_GPIO_ClearPin(PB,_BIT(0));
		msec = 500;
		while(msec);
		HAL_GPIO_SetPin(PB,_BIT(0));
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
