#include "AC30M1x64.h"
#include "AC30M1x64_aa_types.h"
#include "flash.h"

#define	MCLK					(40000000UL)					//40MHz-IOSC40

#define T_CHIP_ERASE			(unsigned int)(MCLK*0.003000)		//3ms
#define T_SECTOR_ERASE		(unsigned int)(MCLK*0.003000)		//3ms
#define T_PGM							(unsigned int)(MCLK*0.003000)		//3ms
#define T_PREPGM					(unsigned int)(MCLK*0.000500)		//500us
#define T_DISCHARGE				(unsigned int)(MCLK*0.000030)		//5us -> 30us	
#define T_AE_DISCHARGE		(unsigned int)(MCLK*0.000030)		//20us -> 30us

#define	FMCFG_ENTRY				0x78580300
#define	FMCFG_EXIT				0x78588300

//====================================================================
// FMMR (Flash Memory Mode Register)
//
//					0x4000_0104
//
//====================================================================
#define BV_IDLE					(0x0001UL<<24)

//====================================================================
// FMCR (Flash Memory Control Register)
//
//					0x4000_0108
//
//====================================================================
#define BV_TIMER				(0x0001UL<<20)
//
#define BV_PGMEVEN				(0x0001UL<<17)
#define BV_PGMODD				(0x0001UL<<16)
//
#define BV_EVER					(0x0001UL<<14)
#define BV_PVER					(0x0001UL<<13)
#define BV_BLKE					(0x0001UL<<12)
//
#define BV_DMYE				(0x0001UL<<11)
#define BV_OTPE					(0x0001UL<<10)
#define BV_AEE					(0x0001UL<<9)
#define BV_AEF					(0x0001UL<<8)
//
#define BV_SUBACT			(0x0001UL<<7)
#define BV_PPGM				(0x0001UL<<6)
#define BV_PMODE				(0x0001UL<<5)
#define BV_WE					(0x0001UL<<4)
//
#define BV_PBLD					(0x0001UL<<3)
#define BV_PGM					(0x0001UL<<2)
#define BV_ERS					(0x0001UL<<1)
#define BV_PBR					(0x0001UL<<0)


void	__WAIT(void)
{
	int	i;

	for(i=0;i<32;i++) {
		__DSB();
		if ((FM->MR&(1<<24)) == 0) 
			break; 
	}
	
	for(i=0; i<100000; i++) {
		// wait until FMMR_IDLE is "1"
		if ((FM->MR&(1<<24)) != 0) 
			break; 
	}
}

void	__DISCHARGE(unsigned long time)
{
    FM->TMR = time;
    FM->CR|= BV_TIMER;	// = 1;
	__WAIT();
    FM->CR|=~(BV_TIMER);	// = 0; 
}

void	__FLASH_ALL_ENABLE(void)
{
	FM->MR = 0x66;
	FM->MR = 0x99;
	FM->WPROT = 0x87000000;	//All Enable
	FM->WPROT = 0x98000000;
}

void	__FLASH_ENABLE(int addr)
{
	FM->MR = 0x66;
	FM->MR = 0x99;
	FM->WPROT = 0x87000000;	//All Enable
	FM->WPROT = 0x98000000;
}

void	__FLASH_DISABLE(void)
{
	FM->MR = 0x66;
	FM->MR = 0x99;
	FM->WPROT = 0x8700FFFF;	//All Enable
	FM->WPROT = 0x9800FFFF;
}

void	__FLASH_ENTRY(void)
{
	//---------------------------------------------------------------------------------
	// 2. enable program mode 
	//---------------------------------------------------------------------------------
	FM->MR = 0x81;
	FM->MR = 0x28;
	FM->CFG = FMCFG_ENTRY;
}

void	__FLASH_ENTRY_PMOD(void)
{
	__FLASH_ENTRY();
	FM->MR = 0x00;

	FM->MR = 0x5A;
	FM->MR = 0xA5;
	FM->CR = BV_PMODE | BV_SUBACT; ///sub


	__WAIT();
}

void	__FLASH_EXIT(void)
{
	FM->CR = 0x00000000;

	FM->CFG = FMCFG_EXIT; 
	FM->MR  = 0x00000000;

	__DSB();
	__DSB();
	__DSB();
	__DSB();
	__DSB();
	__DSB();
}


void	__FLASH_PGM(void)
{
	unsigned long fmcr;
	
	fmcr=FM->CR;
	
	FM->CR = fmcr|=BV_PGM;		// = 1;
	FM->CR = fmcr|=BV_WE;		// = 1;
	__WAIT();
	//FMCR_WEN = 0;
	//FMCR_PGM = 0;
	FM->CR = 0x00000000;
	__DISCHARGE(T_DISCHARGE);
}

//-------------------------------------------------------

int	flash_erase_macro(int dsel)
{
	unsigned long fmcr;
	
	__FLASH_ALL_ENABLE();

#if 1		//PrePGM	
	__FLASH_ENTRY_PMOD();
	fmcr=FM->CR;

	FM->TMR = T_PREPGM;			// 0.5ms = 500us
	FM->CR = fmcr|=BV_PGMODD;	// = 1;
	FM->CR = fmcr|=BV_PGMEVEN;	//= 1;
	FM->CR = fmcr|=BV_AEE;
	__FLASH_PGM();				// Bulk PGM
	__FLASH_EXIT();
#endif

	__FLASH_ENTRY_PMOD();
	fmcr=FM->CR;

	FM->TMR = T_CHIP_ERASE;		// 3.0ms = 3000us
	FM->CR = fmcr|=BV_AEF;		//  = 1;
	FM->CR = fmcr|=BV_ERS;		// = 1;
	FM->CR = fmcr|=BV_WE;		// = 1; 
	__WAIT();					// Chip Erase
	FM->CR = fmcr&=~BV_WE;		// = 0;	
	FM->CR = fmcr&=~BV_ERS;		// = 0;
	__DISCHARGE(T_AE_DISCHARGE);
	FM->CR = fmcr&=~BV_AEF;		//  = 0; 
	
	__FLASH_EXIT();
	__FLASH_DISABLE();
	
	return 0;
}

int	flash_erase_sector(int dsel, unsigned long addr)
{
	unsigned long fmcr;
	
	__FLASH_ENABLE(addr);
	
	__FLASH_ENTRY_PMOD();
	fmcr=FM->CR;

	FM->AR = addr>>2;
	FM->TMR = T_PREPGM;				// 0.5ms = 500us
	FM->CR = fmcr|=BV_AEE;		// = 1;
	__FLASH_PGM();
	__FLASH_EXIT();

	__FLASH_ENTRY_PMOD();
	FM->AR = addr>>2;
	FM->TMR = T_SECTOR_ERASE;			// 3.0ms = 3000us
	FM->CR = fmcr|=BV_ERS;		// = 1;	
	FM->CR = fmcr|=BV_WE;		// = 1;
	__WAIT();
	//CLR_FMCR_WEN;		// = 0;
	//CLR_FMCR_ERS;		// = 0;	
	__DISCHARGE(T_DISCHARGE);
	
	__FLASH_EXIT();
	__FLASH_DISABLE();
	
	return 0;
}

int	flash_write_sector(int dsel, unsigned long addr,unsigned long *pdata)
{
	int	i;
	unsigned long *ptr;
	unsigned long fmcr;
	
	//---------------------------------------------------------------------------------
	// check address and size alignment
	//---------------------------------------------------------------------------------

	ptr = (unsigned long*)pdata;

	__FLASH_ENABLE(addr);

	__FLASH_ENTRY_PMOD();
	fmcr=FM->CR;
	
	FM->AR = addr>>2;
	FM->CR = fmcr|=BV_PBR;		//=1;
	FM->CR = fmcr&=~BV_PBR;		//=0;

	FM->AR = addr>>2;
    FM->CR = fmcr|=BV_PBLD;		// = 1;
    for(i=0; i<128;i+=4)
        FM->DR = *ptr++;
    FM->CR = fmcr&=~BV_PBLD;		// = 0;

	FM->AR = addr>>2;
	FM->TMR = T_PGM;
	__FLASH_PGM();
	
	__FLASH_EXIT();
	__FLASH_DISABLE();
	return 0;
}

