/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void FRT_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void delay(void);
void GPIO_Configure(void);
void FRT_Configure(void);

void FRT_TestRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" Test \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"Using FRT(Free Running Timer) \n\r"
"************************************************\n\r";
uint32_t msec;
uint32_t matchintcnt;
uint32_t overflowintcnt;
uint32_t period = 0;
/**********************************************************************
 * @brief		SysTick_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void)
{
	if(msec)msec--;
}

/**********************************************************************
 * @brief		FRT_IRQHandler Function
 * @param[in]	none
 * @return 		none
 **********************************************************************/
void FRT_IRQHandler_IT(void)
{
	uint8_t status;
	
//	_DBG("Occured FRT Interrupt\r\n");
	
	status = HAL_FRT_GetStatus(FRT);				// Get FRT Status
//	_DBG("FRT.SR :"); _DBH(status); _DBG("\r\n");
	
	if (status & 0x01){ // match interrupt 
		HAL_FRT_SetStatus(FRT,0x01);
		PB->ODR ^=0x01;
		matchintcnt++;
	}
	if (status & 0x02){ // overflow interrupt 
		HAL_FRT_SetStatus(FRT,0x02);
		PB->ODR ^=0x02;
		overflowintcnt++;
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x1000 ; i++) ;
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, 0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
	
	//Pin setting PB1
	HAL_GPIO_ConfigFunction(PCB, 1, 0);
	HAL_GPIO_ConfigOutput(PCB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 1, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(1));
}


/**********************************************************************
 * @brief		FRT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FRT_Configure(void)
{
	if(HAL_FRT_ModeInit(FRT) != HAL_OK)  
	{
		/* Initialization Error */
    Error_Handler();
	}
		/* FRT Clock Source */
	//HAL_SCU_LSE_ClockConfig(2); //INRINGOSC_EN
	//HAL_FRT_ClockSource(FRT, FRT_LSIOSC);	// Set FRT Clock Source Type  0:FRT_LSIOSC, 1:FRT_EXTOSC, 2:FRT_SUBOSC

	HAL_SCU_HSE_ClockConfig(EXOSC_EN); 
	HAL_SCU_HSE_ClockMonitoring();
	HAL_FRT_ClockSource(FRT, FRT_EXTOSC);	// Set FRT Clock Source Type  0:FRT_LSIOSC, 1:FRT_EXTOSC, 2:FRT_SUBOSC

	//HAL_SCU_LSE_ClockConfig(FRT_SUBOSC); 
	//HAL_SCU_LSE_ClockMonitoring();
	//HAL_FRT_ClockSource(FRT, FRT_SUBOSC);	// Set FRT Clock Source Type  0:FRT_LSIOSC, 1:FRT_EXTOSC, 2:FRT_SUBOSC	
	
	/* FRT Period */
	HAL_FRT_SetPeriod(FRT, 0x00001000);	// Set FRT Period	
	period = HAL_FRT_GetPeriod(FRT);	// Get FRT Period
	_DBG("- FRT Period :"); _DBH32(period); _DBG("\r\n");	

	/* FRT Interrupt */
	NVIC_SetPriority(FRT_IRQn, ((0x01<<1)|0x01));	// SET FRT Interrupt Priority
  NVIC_EnableIRQ(FRT_IRQn);						// ENABLE FRT Interrupt 

}


/**********************************************************************
 * @brief		FRT_TestRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FRT_TestRun(void)
{	
	char cmd;

	uint32_t counter;
	
		_DBG("*");			
		getstring();
	
		
//		InFlag =1;
//		cmd='s';
		
	  if(1) //d	if (InFlag)
		{
			_DBG("\n\r");
			cmd=InData[0];			
			
			switch(cmd)
			{
				case 's':	
					// match interrupt test 		
					_DBG("\r\nmatch  interrupt Test\r\n");				
					HAL_FRT_SetPeriod(FRT, 0x008000);	// Set FRT Period	
					period = HAL_FRT_GetPeriod(FRT);	// Get FRT Period
					_DBG("- FRT Period :"); _DBH32(period); _DBG("\r\n");	

					HAL_FRT_CounterMatchClearDisableBit(FRT, DISABLE);		// Disable CounterMatchClearDisableBit

					counter = 0;
					matchintcnt=0;
					HAL_FRT_MatchInterrupt(FRT, ENABLE);			// FRT Match Interrupt Enable				
					HAL_FRT_Run(FRT, ENABLE);							// FRT Enable				
					HAL_FRT_ClearCounter(FRT);						// Clear(Initialize) FRT Counter Value 		

					while(matchintcnt<10){
						msec=1000;
						while (msec--);

						HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
						while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);  // check read request flag
						HAL_FRT_SetStatus(FRT,0x04); //clear 
						
						counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
						_DBG(" - Count :"); _DBH32(counter); _DBG("\r\n");						
					}

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 
					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Count :"); _DBH32(counter); _DBG("\r\n");

					_DBG("Hold On\r\n");
					HAL_FRT_HoldCounter(FRT, ENABLE);		// Hold on FRT Counter
					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold ON1 Count :"); _DBH32(counter); _DBG("\r\n");
					delay();	

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold ON2 Count :"); _DBH32(counter); _DBG("\r\n");
					
					_DBG("Hold Off\r\n");
					HAL_FRT_HoldCounter(FRT, DISABLE);		// Hold off FRT Counter
					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold OFF1 Count :"); _DBH32(counter); _DBG("\r\n");
					delay();	

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold OFF2 Count :"); _DBH32(counter); _DBG("\r\n");
					
					HAL_FRT_Run(FRT, DISABLE);							// FRT Disable				
					HAL_FRT_MatchInterrupt(FRT, DISABLE);			// FRT Interrupt Disable		
					break;	
					
				case 't':
					// Overflow  interrupt Test
					_DBG("\r\nOverflow  interrupt Test\r\n");
					HAL_FRT_SetPeriod(FRT, 0x00018000);	// Set FRT Period	
					period = HAL_FRT_GetPeriod(FRT);	// Get FRT Period
					_DBG("- FRT Period :"); _DBH32(period); _DBG("\r\n");	

					HAL_FRT_CounterMatchClearDisableBit(FRT, ENABLE);		// Enable CounterMatchClearDisableBit

					counter = 0;
					matchintcnt=0;
					overflowintcnt=0;
					HAL_FRT_MatchInterrupt(FRT, ENABLE);			// FRT Interrupt Enable
					HAL_FRT_OverflowInterrupt(FRT, ENABLE);		// FRT Interrupt Enable				
					HAL_FRT_Run(FRT, ENABLE);							// FRT Enable				
					HAL_FRT_ClearCounter(FRT);						// Clear(Initialize) FRT Counter Value 		

					while(overflowintcnt!=5){
						msec=1000;
						while (msec--);

						HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
						while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
						HAL_FRT_SetStatus(FRT,0x04); //clear 
						
						counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
						_DBG(" - Count :"); _DBH32(counter); _DBG("\r\n");
					}

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 
					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Count :"); _DBH32(counter); _DBG("\r\n");

					_DBG("Hold On\r\n");
					HAL_FRT_HoldCounter(FRT, ENABLE);		// Hold on FRT Counter
					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold ON1 Count :"); _DBH32(counter); _DBG("\r\n");
					delay();	

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold ON2 Count :"); _DBH32(counter); _DBG("\r\n");
					
					_DBG("Hold Off\r\n");
					HAL_FRT_HoldCounter(FRT, DISABLE);		// Hold off FRT Counter
					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 					
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold OFF1 Count :"); _DBH32(counter); _DBG("\r\n");
					delay();	

					HAL_FRT_Counter(FRT, ENABLE);					// FRT Counter read count request Enable
					while ((HAL_FRT_GetStatus(FRT) & 0x04)==0);
					HAL_FRT_SetStatus(FRT,0x04); //clear 
					counter = HAL_FRT_GetCounterVal(FRT);		// Get FRT Counter Value	
					_DBG(" - Hold OFF2 Count :"); _DBH32(counter); _DBG("\r\n");
					
					HAL_FRT_Run(FRT, DISABLE);							// FRT Disable				
					HAL_FRT_MatchInterrupt(FRT, DISABLE);			// FRT Interrupt Disable	
					HAL_FRT_OverflowInterrupt(FRT, DISABLE);					
					break;					
				case 'i':
					HAL_FRT_MatchInterrupt(FRT, ENABLE);			// FRT Match Interrupt Enable
					break;
				case 'k':
					HAL_FRT_MatchInterrupt(FRT, DISABLE);			// FRT Match Interrupt Disable
					break;			
				case 'o':
					HAL_FRT_OverflowInterrupt(FRT, ENABLE);		// FRT overflow Interrupt Enable
					break;
				case 'q':
					HAL_FRT_OverflowInterrupt(FRT, DISABLE);		// FRT overflow Interrupt Disable
					break;				
				case 'a':
					HAL_FRT_CounterMatchClearDisableBit(FRT, ENABLE);		// Enable CounterMatchClearDisableBit
					break;
				case 'c':
					HAL_FRT_CounterMatchClearDisableBit(FRT, DISABLE);		// Disable CounterMatchClearDisableBit
					break;
			}		
		}		
		else 
		{
			#ifdef _DEBUG_MSG
			_DBG("\n\r");
			#endif
			//d InFlag=0;
			//d InCount=0;	
		}
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure FRT peripheral*/
	FRT_Configure();

	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		FRT_TestRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
