/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void GPIO_Configure(void);
void SysTick_Configure(void);

void GPIO_ExtIntRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

void delay (void);

void GPIOBE_IRQHandler_IT(void);
void GPIOBO_IRQHandler_IT(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"GPIO External Interrupt demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
" This example used to configure the\n\r "
" GPIO PBx6,7 as source of external Interrupt \n\r"
"************************************************\n\r";
uint8_t mode_level[]=
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: low level Interrupt \n\r"
	"\t - 2: high level Interrupt \n\r";
uint8_t mode_edge[]=
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: falling edge Interrupt \n\r"
	"\t - 2: rising edge Interrupt \n\r"
	"\t - 3: both edge Interrupt \n\r";

uint32_t msec;
	uint8_t val_level_or_edge, mode;

/**********************************************************************
 * @brief		 PBx even port interrupt Handler.
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOBE_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;
	
	status = HAL_GPIO_GetInterruptStatus(PCB);
	if(status & (3<<(6<<1)))
	{
		HAL_GPIO_EXTI_ClearPin(PCB, status & (3<<(6<<1)));
		for (j= 0; j<5; j++)
		{
			HAL_GPIO_SetPin(PD, _BIT(0));
			delay();
			HAL_GPIO_ClearPin(PD, _BIT(0));
			delay();
		}
	}	
}

/**********************************************************************
 * @brief		 PBx odd port interrupt Handler.
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOBO_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;
	
	status = HAL_GPIO_GetInterruptStatus(PCB);
	if(status & (3<<(7<<1)))
	{
		HAL_GPIO_EXTI_ClearPin(PCB, status & (3<<(7<<1)));
		for (j= 0; j<5; j++)
		{
			HAL_GPIO_SetPin(PD, _BIT(1));
			delay();
			HAL_GPIO_ClearPin(PD, _BIT(1));
			delay();
		}
	}		
}

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/*********************************************************************//**
 * @brief		Delay function
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay (void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x100000; i++) ;
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PD0
	HAL_GPIO_ConfigFunction(PCD, 0, PD0_MUX_PD0);
	HAL_GPIO_ConfigOutput(PCD, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 0, DISABLE);	
	HAL_GPIO_SetPin(PD, _BIT(0));
	
	// Test Pin setting PD1
	HAL_GPIO_ConfigFunction(PCD, 1, PD1_MUX_PD1);
	HAL_GPIO_ConfigOutput(PCD, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 1, DISABLE);
	HAL_GPIO_SetPin(PD, _BIT(1));

	// external interrupt test pin PB7
	HAL_GPIO_ConfigFunction(PCB, 7, PB7_MUX_PB7);
	HAL_GPIO_ConfigOutput(PCB, 7, INPUT);
	HAL_GPIO_ConfigPullup(PCB, 7, ENABLE);

	// external interrupt test pin PB6
	HAL_GPIO_ConfigFunction(PCB, 6, PB6_MUX_PB6);
	HAL_GPIO_ConfigOutput(PCB, 6, INPUT);
	HAL_GPIO_ConfigPullup(PCB, 6, ENABLE);
	
  /* preemption = 1, sub-priority = 1 */
  NVIC_SetPriority(GPIOBE_IRQn, ((0x01<<1)|0x01));	
  NVIC_SetPriority(GPIOBO_IRQn, ((0x01<<1)|0x01));	
	
	NVIC_EnableIRQ(GPIOBE_IRQn);
	NVIC_EnableIRQ(GPIOBO_IRQn);	
}


/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
}
	
/**********************************************************************
 * @brief		GPIO_ExtIntRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_ExtIntRun(void)
{
		/* Get some inputs */
	_DBG("\n\rPlease input character 0:LEVEL or 1: EDGE! \n\r");		
	
	val_level_or_edge = 0;
	
	val_level_or_edge = _DG;
	switch (val_level_or_edge)
	{
		case '0':
			_DBC(val_level_or_edge);
			val_level_or_edge = IER_LEVEL;
			break;			
		case '1': 
			_DBC(val_level_or_edge);
			val_level_or_edge = IER_EDGE;
			break;
		default:
			_DBG("\n\r...Please input character 0:LEVEL or 1: EDGE! \n\r");
			break;
	}
	
	if (val_level_or_edge==IER_LEVEL){
		_DBG(mode_level);
	}
	else {
		_DBG(mode_edge);		
	}
	
	mode = 0;
	mode = _DG;
	if (val_level_or_edge==IER_LEVEL){
		switch (mode)
		{
			case '0':
			case '1': 
			case '2': 					
				_DBC(mode);
				mode = mode -0x30;
				break;
			default:
				_DBG("\n\r...Please input digit from 0 to 2 only! \n\r");
				break;
		}	
	}		
	else {
		switch (mode)
		{
			case '0':
			case '1': 
			case '2': 		
			case '3': 						
				_DBC(mode);
				mode = mode -0x30;
				break;
			default:
				_DBG("\n\r...Please input digit from 0 to 3 only! \n\r");
				break;
		}			
	}

	HAL_GPIO_EXTI_Config(PCB, 7, val_level_or_edge, mode);
	HAL_GPIO_EXTI_Config(PCB, 6, val_level_or_edge, mode);
	
	NVIC_EnableIRQ(GPIOBE_IRQn);
	NVIC_EnableIRQ(GPIOBO_IRQn);

	__enable_irq();
	
	while(1){};
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   		/* Infinite loop */
		GPIO_ExtIntRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
