;/*****************************************************************************
; * @file:    startup_MPS_CM0.s
; * @purpose: CMSIS Cortex-M0 Core Device Startup File 
; *           for the ARM 'Microcontroller Prototyping System' 
; * @version: V1.0
; * @date:    19. Aug. 2009
; *------- <<< Use Configuration Wizard in Context Menu >>> ------------------
; *
; * Copyright (C) 2008-2009 ARM Limited. All rights reserved.
; * ARM Limited (ARM) is supplying this software for use with Cortex-M0 
; * processor based microcontrollers.  This file can be freely distributed 
; * within development tools that are supporting such ARM based processors. 
; *
; * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
; * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
; * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
; * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; *
; *****************************************************************************/


; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              	; Top of Stack
                DCD     Reset_Handler             		; Reset Handler
                DCD     NMI_Handler               		; NMI Handler
                DCD     HardFault_Handler         	; Hard Fault Handler
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     0                        			; Reserved
                DCD     SVC_Handler               		; SVCall Handler
                DCD     0                         			; Reserved
                DCD     0                         			; Reserved
                DCD     PendSV_Handler            	; PendSV Handler
                DCD     SysTick_Handler           		; SysTick Handler

                ; External Interrupts
                DCD     LVDFAIL_IRQHandler			; IRQ0
                DCD     SYSCLKFAIL_IRQHandler          ; IRQ1
                DCD     MOSCFAIL_IRQHandler          	; IRQ2
                DCD     SOSCFAIL_IRQHandler          	; IRQ3                                
                DCD     WDT_IRQHandler						; IRQ4
                DCD     TIMER0_IRQHandler					; IRQ5              
                DCD     TIMER1_IRQHandler					; IRQ6
                DCD     TIMER2_IRQHandler					; IRQ7
                DCD     TIMER3_IRQHandler					; IRQ8
                DCD     FRT_IRQHandler						; IRQ9
                DCD     GPIOAE_IRQHandler					; IRQ10  
                DCD     GPIOAO_IRQHandler				; IRQ11
                DCD     GPIOBE_IRQHandler				; IRQ12
                DCD     GPIOBO_IRQHandler				; IRQ13
                DCD     GPIOCE_IRQHandler				; IRQ14                                                                                
                DCD     GPIOCO_IRQHandler				; IRQ15
                DCD     GPIODE_IRQHandler				; IRQ16
                DCD     GPIODO_IRQHandler				; IRQ17
                DCD     MPWM_IRQHandler				; IRQ18
                DCD     MPWMPROT_IRQHandler		; IRQ19
                DCD     MPWMOVV_IRQHandler			; IRQ20
                DCD     I2C_IRQHandler						; IRQ21
                DCD     SPI_IRQHandler						; IRQ22
                DCD     UART0_IRQHandler					; IRQ23
                DCD		UART1_IRQHandler					; IRQ24
                DCD		ADC_IRQHandler					; IRQ25
                DCD		NULL_IRQHandler					; IRQ26
                DCD     NULL_IRQHandler						; IRQ27
                DCD     NULL_IRQHandler						; IRQ28                                                              
                DCD     NULL_IRQHandler						; IRQ29
                DCD     NULL_IRQHandler						; IRQ30
                DCD     NULL_IRQHandler						; IRQ31


                AREA    |.text|, CODE, READONLY


; Reset Handler

;Reset_Handler   PROC
;		EXPORT  Reset_Handler             	[WEAK]
;		IMPORT  __main
;		LDR     R0, =__main
;		BX      R0
;		ENDP

Reset_Handler   PROC
                EXPORT  Reset_Handler             	[WEAK]
                IMPORT  SystemInit
				IMPORT  __main
				LDR     R0, =SystemInit
				BLX     R0
                LDR		R0, =__main
                BX		R0
                ENDP
					

; Dummy Exception Handlers (infinite loops which can be modified)

NMI_Handler     PROC
		EXPORT  NMI_Handler               		[WEAK]
		B       .
		ENDP
HardFault_Handler		PROC
		EXPORT  HardFault_Handler         	[WEAK]
		B       .
		ENDP
SVC_Handler     PROC
		EXPORT  SVC_Handler               	[WEAK]
		B       .
		ENDP
PendSV_Handler  PROC
		EXPORT  PendSV_Handler            	[WEAK]
		B       .
		ENDP
SysTick_Handler PROC
		EXPORT  SysTick_Handler           	[WEAK]
		B       .
		ENDP

NULL_IRQHandler PROC
		EXPORT  LVDFAIL_IRQHandler			[WEAK]
		EXPORT  SYSCLKFAIL_IRQHandler			[WEAK]
		EXPORT  MOSCFAIL_IRQHandler			[WEAK]
		EXPORT  SOSCFAIL_IRQHandler			[WEAK]		
		EXPORT  WDT_IRQHandler						[WEAK]
		EXPORT  TIMER0_IRQHandler					[WEAK]
		EXPORT  TIMER1_IRQHandler					[WEAK]
		EXPORT  TIMER2_IRQHandler					[WEAK]
		EXPORT  TIMER3_IRQHandler					[WEAK]                
		EXPORT  FRT_IRQHandler						[WEAK]
		EXPORT  GPIOAE_IRQHandler					[WEAK]
		EXPORT  GPIOAO_IRQHandler					[WEAK]
		EXPORT  GPIOBE_IRQHandler					[WEAK]
		EXPORT  GPIOBO_IRQHandler					[WEAK]
		EXPORT  GPIOCE_IRQHandler					[WEAK]
		EXPORT  GPIOCO_IRQHandler					[WEAK]
		EXPORT  GPIODE_IRQHandler					[WEAK]	
		EXPORT  GPIODO_IRQHandler					[WEAK]					
		EXPORT  MPWM_IRQHandler					[WEAK]		
		EXPORT  MPWMPROT_IRQHandler		[WEAK]		
		EXPORT  MPWMOVV_IRQHandler			[WEAK]	
		EXPORT  I2C_IRQHandler						[WEAK]		
		EXPORT  SPI_IRQHandler						[WEAK]
		EXPORT  UART0_IRQHandler					[WEAK]
		EXPORT  UART1_IRQHandler					[WEAK]
		EXPORT  ADC_IRQHandler						[WEAK]		
		
LVDFAIL_IRQHandler
SYSCLKFAIL_IRQHandler
MOSCFAIL_IRQHandler
SOSCFAIL_IRQHandler
WDT_IRQHandler
TIMER0_IRQHandler
TIMER1_IRQHandler
TIMER2_IRQHandler
TIMER3_IRQHandler
FRT_IRQHandler 
GPIOAE_IRQHandler
GPIOAO_IRQHandler
GPIOBE_IRQHandler
GPIOBO_IRQHandler
GPIOCE_IRQHandler
GPIOCO_IRQHandler
GPIODE_IRQHandler
GPIODO_IRQHandler
MPWM_IRQHandler
MPWMPROT_IRQHandler
MPWMOVV_IRQHandler
I2C_IRQHandler
SPI_IRQHandler
UART0_IRQHandler
UART1_IRQHandler
ADC_IRQHandler

                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
