/**********************************************************************
* @file		AC30M1x64_PortInit.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Port_Init(void);
/* Private variables ---------------------------------------------------------*/




/**********************************************************************
 * @brief		This function handles NMI exception.
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Port_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	
	
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	PCA->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<30)   // 0:PA15,          1:T3IO                                               
	| (0<<28)   // 0:PA14,          1:T2IO                                                
	| (0<<26)   // 0:PA13,          1:T1IO                                                
	| (0<<24)   // 0:PA12,          1:T0IO                                     
	| (0<<22)   // 0:PA11,                                                          3:AN11                                                                     
	| (0<<20)   // 0:PA10,                                                          3:AN10
	| (0<<18)   // 0:PA9,            1:T3IO,                  2:T1IO,        3:AN9
	| (0<<16)   // 0:PA8,            1:T2IO,                  2:T0IO,        3:AN8
	| (0<<14)   // 0:PA7,            1:T1IO,                                     3:AN7
	| (0<<12)   // 0:PA6,            1:T0IO,                                     3:AN6
	| (0<<10)   // 0:PA5,                                                            3:AN5
	| (0<<8)   // 0:PA4,                                                              3:AN4
	| (0<<6)   // 0:PA3,              1:SCK0                 2:STBO,       3:AN3
	| (0<<4)   // 0:PA2,              1:SS0                   2:WDTO,      3:AN2	
	| (3<<2)   // 0:PA1,              1:T3IO,                                     3:AN1
	| (3<<0)   // 0:PA0,              1:T2IO,                                     3:AN0		
		; 
	PCA->CR = 0 // Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (3<<2)   // P1
	| (3<<0)   // P0	
		; 	
	PCA->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 		
	
	
	PCB->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (1<<14)   // 0:PB7,          1:OVIN0
	| (1<<12)   // 0:PB6,          1:PRTIN0
	| (1<<10)   // 0:PB5,          1:MP0WL
	| (1<<8)   // 0:PB4,          1:MP0WH          
	| (1<<6)   // 0:PB3,          1:MP0VL,          2:MISO0
	| (1<<4)   // 0:PB2,          1:MP0VH,          2:MOSI0
	| (1<<2)   // 0:PB1,          1:MP0UL,          2:SCK0
	| (1<<0)   // 0:PB0,          1:MP0UH,          2:SS0 
		;	
	PCB->CR = 0   // Input, output, open drain, analog
	| (2<<14)   // P7   0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCB->PCR = 0 // pullup (0,1)
	| (0<<7)   // P7  0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	
	PCC->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (1<<30)   // 0:PC15,          1:TXD0
	| (1<<28)   // 0:PC14,          1:RXD0
	| (3<<26)   // 0:PC13,        1:T2IO,                                           3:XOUT
	| (3<<24)   // 0:PC12,        1:T3IO,                                           3:XIN
	| (0<<22)   // 0:PC11,                                      2:T0IO
	| (1<<20)   // 0:PC10,        1:nRESET
	| (1<<18)   // 0:PC9,          1:CLKO
	| (0<<16)   // 0:PC8,          1:SDA0
	| (0<<14)   // 0:PC7,          1:SCL0,                  2:T3IO
	| (0<<12)   // 0:PC6,          1:TXD1,                   2:T2IO
	| (0<<10)   // 0:PC5,          1:RXD1,                  2:T1IO
	| (0<<8)   // 0:PC4,          1:nTRST,                  2:T0IO
	| (0<<6)   // 0:PC3
	| (0<<4)   // 0:PC2
	| (1<<2)   // 0:PC1,          1:SWDIO,                  2:TXD1
	| (1<<0)   // 0:PC0,          1:SWCLK,                 2:RXD1
		;		
	PCC->CR = 0   // Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2<<28)   // P14
	| (3<<26)   // P13  if XOUT, it must be Analog 
	| (3<<24)   // P12  if XIN, it must be Analog
	| (0<<22)   // P11
	| (2<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (2<<2)   // P1
	| (2<<0)   // P0	
		; 	
	PCC->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 

	PCD->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<6)   // 0:PD3,          1:MISO0,           2:SDA0,          3:SXIN
	| (0<<4)   // 0:PD2,          1:MOSI0,           2:SCL0,          3:SXOUT
	| (0<<2)   // 0:PD1,          1:SCK0,           2:RXD0
	| (0<<0)   // 0:PD0,          1:SS0,             2:TXD0
		;		
	PCD->CR = 0   // Input, output, open drain, analog
	| (0<<6)   // P3  0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCD->PCR = 0 // pullup (0,1)
	| (0<<3)   // P3 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}


