/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void	NMI_Handler_IT(void);
void MPWMPROT_IRQHandler_IT(void);
void	MPWMOVV_IRQHandler_IT(void);
void	MPWM_IRQHandler_IT(void);
void ADC_IRQHandler_IT(void);

void	check_prot(void);
void adc_init(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void MPWM_Configure(void);
void MPWM_Run(void);

void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Motor PWM interrupt demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using MPWM\n\r"
"************************************************\n\r";

static	int	_prot0 = 0;
static	int	_prot1 = 0;
uint32_t fflag;
uint16_t adcval[8];
	char cmd;
	char ch;
	uint16_t dutyh;
	uint16_t dutyl;
	uint16_t delta=0x10;
	int	trg;	
	int	mode;
/**********************************************************************
 * @brief		NMI_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	NMI_Handler_IT(void)
{
	int	sts;
	
	sts = SCU->NMIR;
	if(sts & (1<<12)) { //BV_MP0PR0STS
		MP->PSR0 = 0xCA80;
		_prot0+= 100;
	}
	if(sts & (1<<11)) { //BV_MP0PR1STS
		MP->PSR1 = 0xAC80;
		_prot1+= 100;
	}
}

/**********************************************************************
 * @brief		MPWMPROT_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void MPWMPROT_IRQHandler_IT(void)
{
	MP->PSR0 = 0xCA80;
	_prot0++;
}

/**********************************************************************
 * @brief		MPWMOVV_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWMOVV_IRQHandler_IT(void)
{
	MP->PSR1 = 0xAC80;
	_prot1++;
}


/**********************************************************************
 * @brief		MPWM_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM_IRQHandler_IT(void)
{
	int	irq;
	irq=MP->SR;// & MP0IER;
	//MP0SR = 0xFF;	//irq;
	
	if(irq&(1<<7)) { //BV_PRDIF
		MP->SR = (1<<7);
		PA->ODR^=(1<<7);		
	}
	if(irq&(1<<6)) { //BV_BOTIF
		MP->SR = (1<<6);
		PA->ODR^=(1<<6);			
	}
	if(irq&(1<<5)) { //BV_WHIF
		MP->SR = (1<<5);
		PA->ODR^=(1<<5);			
	}
	if(irq&(1<<4)) { //BV_VHIF
		MP->SR = (1<<4);
		PA->ODR^=(1<<4);			
	}
	if(irq&(1<<3)) { //BV_UHIF
		MP->SR = (1<<3);
		PA->ODR^=(1<<3);			
	}
	if(irq&(1<<2)) { //BV_WLIF
		MP->SR = (1<<2);
		PA->ODR^=(1<<2);			
	}
	if(irq&(1<<1)) {  //BV_VLIF
		MP->SR = (1<<1);
		PA->ODR^=(1<<1);			
	}
	if(irq&(1<<0)) { //BV_ULIF
		MP->SR = (1<<0);
		PA->ODR^=(1<<0);			
	}
}


/**********************************************************************
 * @brief		 ADC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(AD);

	if (status & ADC_STAT_SEQ){
		PA->ODR^=(1<<3);
		HAL_ADC_ClearStatus(AD, (1<<2));
		adcval[0] = AD->DR0;
		adcval[1] = AD->DR1;		
		adcval[2] = AD->DR2;		
		AD->CSCR =0; // !! 
	}
	
	if (status & ADC_STAT_SINGLE){
		fflag=1;
		HAL_ADC_ClearStatus(AD, 1);	
		PA->ODR^=(1<<8);
		adcval[0] = AD->DR0;
		adcval[1] = AD->DR1;		
	}	
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		check_prot
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	check_prot(void)
{
	if(_prot0) {
		_DBG("\n\r");
		_DBG("PROT0="); _DBH(_prot0);_DBG("\n\r");		
		_prot0=0;
	}
	if(_prot1) {
		_DBG("\n\r");
		_DBG("PROT1="); _DBH(_prot1);_DBG("\n\r");			
		_prot1=0;
	}
}

/**********************************************************************
 * @brief		adc_init
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void adc_init(void)
{
#if 1	//ADC_SINGLE_MODE
	ADC_CFG_Type AD0_config;

	AD0_config.Mode = ADC_SINGLE_MODE;
	AD0_config.SampingTime=0x1f;
	AD0_config.SeqCnt = 3;
	AD0_config.RestartEn = 1;
	AD0_config.TrgSel = ADC_TRIGGER_MPWM0;
	
	AD0_config.UseClk = ADC_INTERNAL_CLK;
	AD0_config.InClkDiv = 0;//(((SystemPeriClock/1000000UL) * 2)/ (3*(15+AD0_config.SampingTime))) + 1; 
	if( HAL_ADC_Init(AD, &AD0_config) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	AD->CSCR =(0<<4);	

//	ADC_TriggerlSel(AD, 0x00000000); // use ATR1
//	ADC_TriggerlSel(AD, 0x00000010); //use ATR1, ATR2
	HAL_ADC_TriggerlSel(AD, 0x00000210); //use ATR1, ATR2, ATR3	

	HAL_ADC_ChannelSel(AD, 0x00000010);		
	
	HAL_ADC_ClearStatus(AD, 0x0f); // before adc interrupt enable, you must clear adc status!! 
	HAL_ADC_ConfigInterrupt(AD, (ADC_INTEN_SINGLE | ADC_INTEN_SEQ), ENABLE);

	NVIC_SetPriority(ADC_IRQn, 3);
	/* Enable Interrupt for ADC channel */
    NVIC_EnableIRQ(ADC_IRQn);

//	ADC_ChannelSel(AD, 0x00000010);		
#endif

#if 0	//ADC_BURST_MODE
	ADC_CFG_Type AD0_config;

	AD0_config.Mode = ADC_BURST_MODE;
	AD0_config.SampingTime=0x1f;
	AD0_config.SeqCnt = 2;
	AD0_config.RestartEn = 1;
	AD0_config.TrgSel = ADC_TRIGGER_MPWM0;
	
	AD0_config.UseClk = ADC_INTERNAL_CLK;
	AD0_config.InClkDiv = 2;//(((SystemPeriClock/1000000UL) * 2)/ (3*(15+AD0_config.SampingTime))) + 1; 

	if( HAL_ADC_Init(AD, &AD0_config) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	HAL_ADC_TriggerlSel(AD, 0x00000020); //use ATR1, ATR3

	HAL_ADC_IntConfig(AD, ADC_INTEN_SEQ, ENABLE);

	NVIC_SetPriority(ADC0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for ADC0 channel */
  NVIC_EnableIRQ(ADC0_IRQn);

	HAL_ADC_ChannelSel(AD, 0);	
#endif

}

/**********************************************************************
 * @brief		MPWM_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void MPWM_Configure(void)
{
//	SCU_SetMCCRx(2,PWM0_TYPE,CLKSRC_LSI,1); // select clock source
	HAL_SCU_MiscClockConfig(2,PWM0_TYPE,CLKSRC_MCLK,1); // select clock source
//	SCU_SetMCCRx(2,PWM0_TYPE,CLKSRC_HSI,1); // select clock source
//	SCU_SetMCCRx(2,PWM0_TYPE,CLKSRC_EOSC,1); // select clock source
	
	if( HAL_MPWM_Init(MP)!= HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	HAL_MPWM_Cmd(MP,0,1,0,1,0,1);	// MPWM mode reg setting - motor mode
//	MPWM_Cmd(MP,1,1,1,0,0,0);	// MPWM mode reg setting - pwm mode
	
	dutyh=0x8900;
	dutyl=0x300;
	HAL_MPWM_SetPeriod(MP,0xa000);	
	HAL_MPWM_SetUDuty(MP,dutyh,dutyl);
	HAL_MPWM_SetVDuty(MP,dutyh,dutyl);
	HAL_MPWM_SetWDuty(MP,dutyh,dutyl);	
	HAL_MPWM_SetDeadTime(MP,1,0,0,10); // dead time setting 

	MP->ATR1 = (1<<16) | (0x300);
	MP->ATR2 = (1<<16) | (0x4900);	
	MP->ATR3 = (1<<16) | (0x8900);	
	
  /* preemption = 1, sub-priority = 1 */
  NVIC_SetPriority(MPWM_IRQn, 3);
	/* Enable Interrupt for MPWM channel */
  NVIC_EnableIRQ(MPWM_IRQn);

	mode=0;

}
	
/**********************************************************************
 * @brief		MPWM_Run
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void MPWM_Run(void)
{	
		_DBG("*");		
		getstring();
    
		if (InFlag) {
			_DBG("\n\r");
			cmd=InData[0];
      
			if(strncmp(InData,"test",4)==0) {
				_DBG("test ");
				if(strncmp(InData+5,"adc",3)==0) {
					_DBG("adc ");
					adc_init();
					_DBG("\n\rADC ATR test \n\r");
				}					
				_DBG("\n\r");
			}
			else {
				switch(cmd) {
			case 'n':	//halt
						MP->CR2 ^=0x80;
						_DBG("\n\r");
						_DBG("MP->CR2="); _DBH(MP->CR2);_DBG("\n\r");
						break;
			case 'x':	//start
						MP->CR2 ^=1;
						_DBG("\n\r");
						_DBG("MP->CR2="); _DBH(MP->CR2);_DBG("\n\r");
						break;	
			case 'i':	
						ch=_DG;
						switch(ch) {
						case '0':	MP->IER^= 0x01;	break; //BV_ULIE
						case '1':	MP->IER^= 0x02;	break; //BV_VLIE
						case '2':	MP->IER^= 0x04;	break; //BV_WLIE
						case '3':	MP->IER^= 0x08;	break; //BV_UHIE
						case '4':	MP->IER^= 0x10;	break; //BV_VHIE
						case '5':	MP->IER^= 0x20;	break; //BV_WHIE
						case '6':	MP->IER^= 0x40;	break; //BV_BOTIE
						case '7':	MP->IER^= 0x80;	break; //BV_PRDIE
						}
						_DBG("\n\r");
						_DBG("MP->IER="); _DBH(MP->IER);_DBG("\n\r");						
						break;
			case 'q':	ch=_DG;
						switch(ch) {
						case '0': 	MP->PSR0^= 0xCA01;	break;
						case '1': 	MP->PSR0^= 0xCA02;	break;
						case '2': 	MP->PSR0^= 0xCA04;	break;
						case '3': 	MP->PSR0^= 0xCA08;	break;
						case '4': 	MP->PSR0^= 0xCA10;	break;
						case '5': 	MP->PSR0^= 0xCA20;	break;
						case '6': 	MP->PSR0^= 0xCA80;	break;
						}
			case ' ':
						_DBG("\n\r");
						_DBG("PB->IDR="); _DBH(PB->IDR);	
						_DBG("\n\r");
						_DBG("MP->PCR0="); _DBH(MP->PCR0);	
						_DBG("\n\r");
						_DBG("MP->PSR0="); _DBH(MP->PSR0);_DBG("\n\r");							
						break;
			case '1':	delta=1;break;
			case '0':	delta=0x10;break;
			case 's':	if(++mode>3) mode=0;
						MP->MR = (MP->MR&~(3<<1))|(mode<<1);
						_DBG("\n\r");
						_DBG("MCHMOD="); _DBD(mode);_DBG("\n\r");					
						break;

			case 'u':	if(MP->MR&0x0001) {
							MP->MR&=~0x0001;
							_DBG("(Up-count Mode)");_DBG("\n\r");
						}
						else {
							MP->MR|= 0x0001;
							_DBG("(Up-down Mode)");_DBG("\n\r");
						}
						break;						
			case '[':	dutyh-=delta;
						MP->DUH= dutyh;
						MP->DVH= dutyh;
						MP->DWH= dutyh;		
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" dutyh="); _DBD16(dutyh);	_DBG("\n\r");	
						break;
			case ']':	dutyh+=delta;
						MP->DUH= dutyh;
						MP->DVH= dutyh;
						MP->DWH= dutyh;
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" dutyh="); _DBD16(dutyh);_DBG("\n\r");							
						break;
			case '{':	dutyl-=delta;
						MP->DUL= dutyl;
						MP->DVL= dutyl;
						MP->DWL= dutyl;
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" dutyl="); _DBD16(dutyl);_DBG("\n\r");							
						break;
			case '}':	dutyl+=delta;
						MP->DUL= dutyl;
						MP->DVL= dutyl;
						MP->DWL= dutyl;
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" dutyl="); _DBD16(dutyl);_DBG("\n\r");					
						break;
			case ',':	trg-= delta;
						MP->ATR1=trg;
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" trg="); _DBD16(trg);_DBG("\n\r");							
						break;
			case '.':	trg+= delta;
						MP->ATR1=trg;
						_DBG("\n\r");
						_DBG("MP->MR="); _DBH(MP->MR);	
						_DBG(" trg="); _DBD16(trg);_DBG("\n\r");							
						break;						
			case 'd':
						_DBG("\n\r");							
						break;
				}
			}
		}		
		else {
			_DBG("\n\r");
			InFlag=0;
			InCount=0;	
		}
	

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
		
	
	/*MPWM Configure*/
	MPWM_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		MPWM_Run();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
