/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void GPIOBO_IRQHandler_IT(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void delay (void);
void GPIO_Configure(void);
void PWR_SleepConfigure(void);
void PWR_SleepRun(void);

void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Power control demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"This example used to enter system in sleep mode  \n\r"
"and wake up it by using GPIO PB7 pin\n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		 GPIOBO_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/ 
void GPIOBO_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;
	
	status = HAL_GPIO_GetInterruptStatus(PCB);
	if(status & (3<<(7<<1)))
	{
		HAL_GPIO_EXTI_ClearPin(PCB, status & (3<<(7<<1)));
		for (j= 0; j<5; j++)
		{
			HAL_GPIO_SetPin(PD, _BIT(1));
			delay();
			HAL_GPIO_ClearPin(PD, _BIT(1));
			delay();
		}
	}		
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		Delay function
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay (void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x100000; i++) ;
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PD0
	HAL_GPIO_ConfigFunction(PCD, 0, PD0_MUX_PD0);
	HAL_GPIO_ConfigOutput(PCD, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 0, DISABLE);	
	HAL_GPIO_ClearPin(PD, _BIT(0));

	// Test Pin setting PD1
	HAL_GPIO_ConfigFunction(PCD, 1, PD1_MUX_PD1);
	HAL_GPIO_ConfigOutput(PCD, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 1, DISABLE);
	HAL_GPIO_ClearPin(PD, _BIT(1));
	
	// external interrupt test pin PB7
	HAL_GPIO_ConfigFunction(PCB, 7, PB7_MUX_PB7);
	HAL_GPIO_ConfigOutput(PCB, 7, INPUT);
	HAL_GPIO_ConfigPullup(PCB, 7, ENABLE);
	
	// external test pin PB6
	HAL_GPIO_ConfigFunction(PCB, 6, PB6_MUX_PB6);
	HAL_GPIO_ConfigOutput(PCB, 6, INPUT);
	HAL_GPIO_ConfigPullup(PCB, 6, ENABLE);

}

/**********************************************************************
 * @brief		PWR_SleepConfigure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SleepConfigure(void)
{
	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(GPIOBO_IRQn, ((0x01<<1)|0x01));	

	HAL_GPIO_EXTI_Config(PCB, 7, IER_EDGE, 1);
	HAL_SCU_WakeUpSRCCmd(WAKEUP_GPIOB, ENABLE);
	NVIC_EnableIRQ(GPIOBO_IRQn);
	//	NVIC_ClearPendingIRQ(GPIOBO_IRQn);

	//	__disable_irq();
	//	HAL_GPIO_EXTI_ClearPin(PCB, 3<<(7<<1));
		__enable_irq();

    _DBG_("Press PB6 to enter powerdown mode!");
	while(HAL_GPIO_ReadPin(PB) & (1UL<<6)) {}
    
    _DBG("\n\rEnter sleep...connect PB7 pin to GND to exit sleep mode");
    HAL_PWR_EnterSleepMode();
    _DBG("\n\rWaked up from sleep");
}




/**********************************************************************
 * @brief		PWR_SleepRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SleepRun(void)
{	
		delay();
		HAL_GPIO_SetPin(PD, _BIT(0));
		delay();
		HAL_GPIO_ClearPin(PD, _BIT(0));
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();

	/*Configure Sleep*/
	PWR_SleepConfigure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
  
	while(1)
	{
  		 /* Infinite loop */
		PWR_SleepRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
