/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TEST_SINGLE_POLLING // spi master polling : single data 
//#define TEST_SINGLE_INTERRUPT // spi master interrupt : single data 
//#define TEST_MULTI_INTERRUPT // spi master interrupt : multi data 10 bytes

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void GPIO_Configure(void);
void SysTick_Configure(void);

void GPIO_LedBlinkyRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" SPI master demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"  \n\r"
"************************************************\n\r";
uint32_t rxdata;
uint32_t tmp;
uint32_t i;
SPI_CFG_Type SP_config;


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
// MASTER : SPI0(SS0, SCK0, MOSI0, MISO0) PIN FUNCTION SETTING
	HAL_GPIO_ConfigFunction(PCD, 0, PD0_MUX_SS0);
	HAL_GPIO_ConfigOutput(PCD, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 0, DISABLE);
	
	HAL_GPIO_ConfigFunction(PCD, 1, PD1_MUX_SCK0);
	HAL_GPIO_ConfigOutput(PCD, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 1, DISABLE);
	
	HAL_GPIO_ConfigFunction(PCD, 2, PD2_MUX_MOSI0);
	HAL_GPIO_ConfigOutput(PCD, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCD, 2, DISABLE);
	
	HAL_GPIO_ConfigFunction(PCD, 3, PD3_MUX_MISO0);
	HAL_GPIO_ConfigOutput(PCD, 3, INPUT);
	HAL_GPIO_ConfigPullup(PCD, 3, ENABLE);
}

/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_Configure(void)
{

	SP_config.Databit = SPI_DS_8BITS;
	SP_config.CPHA = SPI_CPHA_LO;
	SP_config.CPOL = SPI_CPOL_LO;
	SP_config.DataDir = SPI_MSB_FIRST;
	SP_config.Mode = SPI_MASTER_MODE;
	SP_config.BaudRate = 79; // PCLK / (79+1) 
	
	HAL_SPI_Init(SP, &SP_config);
	HAL_SPI_Enable(SP, ENABLE);  // SPI enable 
	
	i=0x30;
	_DG;	
}
	
/**********************************************************************
 * @brief		SPI_MasterPollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_MasterPollingRun(void)
{	
	while((HAL_SPI_GetStatus(SP) & SPI_STAT_TXBUF_EMPTY) != SPI_STAT_TXBUF_EMPTY); // check tx buffer empty 
	HAL_SPI_TransmitData(SP, i);

	_DBG(" tx= ");_DBH(i);
	
	i++;
	if (i==0x3a)
	{
		i=0x30;
		_DBG("\r\n");
		_DG;
	}
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SPI Configure*/
	SPI_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   		/* Infinite loop */
		SPI_MasterPollingRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
