/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER0_IRQHandler_IT(void);
void GPIOAE_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);

void GPIO_Configure(void);
void TIMER_CaptureConfigure(void);
void TIMER_CaptureGPIOPrint(void);
void TIMER_CaptureRun(void);

void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Timer Capture demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using Timer 0 \n\r"
"************************************************\n\r";


TIMER_CAPTURECFG_Type T0_Config;
__IO	uint32_t count;
__IO	uint32_t flag;
__IO	uint32_t fflag;
/**********************************************************************
 * @brief		TIMER0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((TIMER_GetStatus(T0) & TIMER_SR_MFA) == TIMER_SR_MFA){
		HAL_TIMER_ClearStatus(T0, TIMER_SR_MFA);
		count++;
		if (count >= 1000){
			count = 0;
			if (flag==0){
				flag=1;
				HAL_GPIO_ClearPin(PD, _BIT(0));					
			}
			else {
				flag=0;
				HAL_GPIO_SetPin(PD, _BIT(0));
			}				
		}	
	}
}

/**********************************************************************
 * @brief		GPIOAE_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAE_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_GPIO_GetInterruptStatus(PCA);
	if(status & (3<<(6<<1)))
	{
		HAL_GPIO_EXTI_ClearPin(PCA, status & (3<<(6<<1)));
		if (fflag == 0){
			fflag=1;
		}
		else if (fflag ==1){
			fflag = 2;
		}
	}	
}
/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
   // Test Pin setting PA6
	HAL_GPIO_ConfigFunction(PCA, 6, PA6_MUX_T0IO);
	HAL_GPIO_ConfigOutput(PCA, 6, INPUT);
	HAL_GPIO_ConfigPullup(PCA, 6, DISABLE);	


	// pulse out pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
}


/**********************************************************************
 * @brief		TIMER_CaptureConfigure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_CaptureConfigure(void)
{

  T0_Config.CkSel = PCLK_2; 	// SystemPeriClock=40Mhz=PCLK  CLOCK_IN = PCLK/2=20Mhz 
	T0_Config.Prescaler = 19;    // TCLK = (CLOCK_IN / (PRS+1))  = 20Mhz / 20 = 1Mhz ->1us

	T0_Config.ClrMode=RISING_EGDE;
	T0_Config.AdcTrgEn=DISABLE;
	
 	 if(HAL_TIMER_Init(T0, CAPTURE_MODE, &T0_Config) != HAL_OK)
	{
		/* Initialization Error */
   		 Error_Handler();
	}
  //HAL_TIMER_ConfigInterrupt(T0, TIMER_INTCFG_OVIE, ENABLE);

	/* preemption = 1, sub-priority = 1 */
	//NVIC_SetPriority(TIMER0_IRQn, 3);
	/* Enable Interrupt for TIMER0 channel */
	//NVIC_EnableIRQ(TIMER0_IRQn);

  	NVIC_SetPriority(GPIOAE_IRQn, 3);	
	HAL_GPIO_EXTI_Config(PCA, 6, IER_EDGE, 3); 	
	NVIC_EnableIRQ(GPIOAE_IRQn);
	
	fflag=0;
	
	HAL_TIMER_ClearCounter(T0);
	HAL_TIMER_Cmd(T0, ENABLE); // timer start

}
/**********************************************************************
 * @brief		TIMER_CaptureGPIOPrint
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_CaptureGPIOPrint(void)
{
	uint32_t i;
	
	_DBG("\n\r Wait Pulse! \n\r");
	
	HAL_GPIO_ClearPin(PB, _BIT(0));
	for (i=0;i<1000;i++);
	HAL_GPIO_SetPin(PB, _BIT(0));
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);		
	HAL_GPIO_ClearPin(PB, _BIT(0));
	for (i=0;i<1000;i++);
	HAL_GPIO_SetPin(PB, _BIT(0));

}
	

/**********************************************************************
 * @brief		TIMER_CaptureRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_CaptureRun(void)
{
	switch(fflag){
		case 0:
			break;
		case 1:
			break;
		case 2:
			_DBG("capture value! \n\r");
			count = HAL_TIMER_GetCountValue(T0,1); 
			_DBG("GRA = "); _DBH16(count); _DBG_(" ");
			fflag=0;
			break;
	}
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure TIMER peripheral*/
	TIMER_CaptureConfigure();

	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*TIMER GPIO Print*/
	TIMER_CaptureGPIOPrint();

	while(1)
	{
            /* Infinite loop */
            TIMER_CaptureRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
