/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER3_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure(void);
void TIMER_Cmd(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Timer Match interrupt demo \n\r"
"\t - MCU: AC30M1464 \n\r"
"\t - Core: ARM Cortex-M0 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using Timerx in interrupt mode\n\r"
"************************************************\n\r";
TIMER_PERIODICCFG_Type T3_Config;
__IO	uint32_t count;
__IO	uint32_t flag;
/**********************************************************************
 * @brief		TIMER3_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER3_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = TIMER_GetStatus(T3);
	if ((status & TIMER_SR_MFB) == TIMER_SR_MFB){
		//HAL_TIMER_ClearStatus(Tx, TIMER_SR_MFB);
		count++;
		if (count >= 1000){
			count = 0;
			if (flag==0){
				flag=1;
				HAL_GPIO_ClearPin(PB, _BIT(0));					
			}
			else {
				flag=0;
				HAL_GPIO_SetPin(PB, _BIT(0));
			}				
		}	
	}
	HAL_TIMER_ClearStatus(T3, status);
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));
	
	// T0O pin config set if need
	//HAL_GPIO_ConfigFunction(PCA, 6, PA6_MUX_T0IO);
	//HAL_GPIO_ConfigOutput(PCA, 6, PUSH_PULL_OUTPUT);
	//HAL_GPIO_ConfigPullup(PCA, 6, DISABLE);
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	
	T3_Config.CkSel = PCLK_2; // SystemPeriClock=40Mhz=PCLK  CLOCK_IN = PCLK/2=20Mhz 
	T3_Config.Prescaler = 19;    // TCLK = (CLOCK_IN / (PRS+1))  = 20Mhz / 20 = 1Mhz ->1us

	T3_Config.GRA = (1000); 	
	T3_Config.GRB = (1000); // 1msec			
	T3_Config.Cnt=0;		
	T3_Config.StartLevel=START_LOW;
	T3_Config.AdcTrgEn=DISABLE;
	
	if( HAL_TIMER_Init(T3, PERIODIC_MODE, &T3_Config) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	HAL_TIMER_ConfigInterrupt(T3, TIMER_INTCFG_MBIE, ENABLE);

	NVIC_SetPriority(TIMER3_IRQn, 3);
	/* Enable Interrupt for TIMERx channel */
	NVIC_EnableIRQ(TIMER3_IRQn);


}
/**********************************************************************
 * @brief		TIMER_Cmd
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Cmd(void)
{
	HAL_TIMER_Cmd(T3, ENABLE); // timer start
}
	



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*TIMER Configure*/
	TIMER_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*TIMER Start*/
	TIMER_Cmd();

	while(1)
	{
   		/* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
