/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
//#define SET_CONST
#ifdef SET_CONST
const UART_CFG_Type UARTConfigStruct=
{
	38400,
	UART_DATABIT_8,
	UART_PARITY_NONE,
	UART_STOPBIT_1
};
#endif

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void GPIO_Configure(void);
void UART_Configure(void);
void UART_Transmit(void);
void UART_PollingRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
UART_CFG_Type UARTConfigStruct;



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	//UART0 pin config setting in PCU
	HAL_GPIO_ConfigFunction(PCC, 14, PC14_MUX_RXD0);
	HAL_GPIO_ConfigOutput(PCC, 14, INPUT);
	HAL_GPIO_ConfigPullup(PCC, 14, ENABLE);
	
	HAL_GPIO_ConfigFunction(PCC, 15, PC15_MUX_TXD0);
	HAL_GPIO_ConfigOutput(PCC, 15, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PCC, 15, ENABLE);	

	//UART1 pin config setting in PCU
	HAL_GPIO_ConfigFunction(PCC, 5, PC5_MUX_RXD1);
	HAL_GPIO_ConfigOutput(PCC, 5, INPUT);
	HAL_GPIO_ConfigPullup(PCC, 5, ENABLE);
	
	HAL_GPIO_ConfigFunction(PCC, 6, PC6_MUX_TXD1);
	HAL_GPIO_ConfigOutput(PCC, 6, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PCC, 6, ENABLE);	
	
}


/**********************************************************************
 * @brief		UART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{
	// default: 38400-8-N-1 
	HAL_UART_ConfigStructInit(&UARTConfigStruct);

	if(HAL_UART_Init(U0,(UART_CFG_Type *)(&UARTConfigStruct)) != HAL_OK)  
	{
		/* Initialization Error */
    		Error_Handler();
	}

}
/**********************************************************************
 * @brief		UART_Transmit
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Transmit(void)
{
	HAL_UART_Transmit(U0,(uint8_t *)"Polling test \n\r",15,BLOCKING);
}

	

/**********************************************************************
 * @brief		UART_PollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_PollingRun(void)
{
	uint8_t ch;

	HAL_UART_Receive(U0,&ch,1,BLOCKING);
	HAL_UART_Transmit(U0, &ch,1,BLOCKING);
		
	if (ch==0x0d) 
	{
		HAL_UART_Transmit(U0,(uint8_t *)"\n\r",2,BLOCKING);
	}	
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure UART peripheral*/
	UART_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/* UART  Start*/
	UART_Transmit();

	while(1)
	{
  		 /* Infinite loop */
		UART_PollingRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
