/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void WDT_IRQHandler_IT(void);
void NMI_Handler_IT(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WDT_Configure(void);
void SysTick_Configure(void);
void WDT_ResetRun(void);
void WDT_Strat(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
	"************************************************\n\r"
	"Watch dog timer interrupt demo\n\r"
	"\t - MCU: AC30M1464 \n\r"
	"\t - Core: ARM Cortex-M0 \n\r"
	"\t - Communicate via: UART0 - 38400 bps \n\r"
	"************************************************\n\r";
uint8_t wdtf;
WDT_CFG_Type wdtCfg;

/**********************************************************************
 * @brief		WDT_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void WDT_IRQHandler_IT(void)
{
#if 0	// 0: PCLK, 1:external from MCCR3 
		HAL_WDT_ReloadTimeCounter(40000);
#else 
		HAL_WDT_ReloadTimeCounter(10000000);	
#endif
		if (wdtf == 0){
			wdtf=1;
			HAL_GPIO_ClearPin(PB, _BIT(0));
		}
		else {
			wdtf=0;
			HAL_GPIO_SetPin(PB, _BIT(0));
		}
}

/**********************************************************************
 * @brief		NMI_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void NMI_Handler_IT(void)
{
		uint32_t status;
		
		status = HAL_SCU_GetNMIStatus();
		
		if ( ((status&(1<<10)) == (1<<10)) && ((status&1<<2)==(1<<2)) ){ //wdt nmi en
			HAL_WDT_ReloadTimeCounter(40000);
			if (wdtf == 0){
				wdtf=1;
				HAL_GPIO_ClearPin(PB, _BIT(0));
			}
			else {
				wdtf=0;
				HAL_GPIO_SetPin(PB, _BIT(0));
			}
		}
	}



/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));

}


/**********************************************************************
 * @brief		WDT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Configure(void)
{

	WDT_CFG_Type wdtCfg;
	__IO uint32_t wdtcnt;
	
	if(HAL_WDT_Init() != HAL_OK)  
	{
		/* Initialization Error */
    		Error_Handler();
	}
	//	HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_LSI, 1);
		HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_MCLK, 1);
	//	HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_HSI, 1);
	//	HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_EOSC, 1);
	//	HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_SOSC, 1);
		
		wdtCfg.wdtResetEn = DISABLE;
		wdtCfg.wdtCountEn = ENABLE;
#if 0 // 0: PCLK, 1:MCCR3 
		wdtCfg.wdtClkSel = SET; //SET:external from MCCR3 , RESET:PCLK
		wdtCfg.wdtPrescaler = WDT_DIV_1;  //WDTCLK = WDTCLKIN / PRS
		wdtCfg.wdtTmrConst = 40000; //1s
#else 
		wdtCfg.wdtClkSel = RESET;
		wdtCfg.wdtPrescaler = WDT_DIV_4;  //WDTCLK = WDTCLKIN / PRS =  40Mhz / 4 
		wdtCfg.wdtTmrConst = 10000000; //1s
#endif
		
		HAL_WDT_Configure(wdtCfg);
		
#if 1  // 1: wdt interrupt mode , 0: wdt nmi interrupt mode 
		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(WDT_IRQn, ((0x01<<1)|0x01));
		/* Enable Interrupt for TIMER0 channel */
		NVIC_EnableIRQ(WDT_IRQn);
#else 
		HAL_SCU_SetNMI(1<<2);
		NVIC_EnableIRQ(NonMaskableInt_IRQn);
#endif


}
/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt

}
	

/**********************************************************************
 * @brief		WDT_InterruptStart
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_InterruptStart(void)
{
	HAL_WDT_Start(ENABLE);
}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure WDT peripheral*/
	WDT_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();
  
	/*WDT Start*/
	WDT_InterruptStart();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
    		/* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
