/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WDT_Configure(void);
void SysTick_Configure(void);
void WDT_ResetRun(void);
void WDT_Strat(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
	"************************************************\n\r"
	"Watch dog timer Reset demo\n\r"
	"\t - MCU: AC30M1464 \n\r"
	"\t - Core: ARM Cortex-M0 \n\r"
	"\t - Communicate via: UART0 - 38400 bps \n\r"
	"************************************************\n\r";

uint32_t msec;
WDT_CFG_Type wdtCfg;
__IO uint32_t wdtcnt;
uint8_t i,j;

/**********************************************************************
 * @brief		SysTick_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 1000000 ; i++) ;
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigFunction(PCB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PCB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PCB, 0, DISABLE);	
	HAL_GPIO_SetPin(PB, _BIT(0));

}


/**********************************************************************
 * @brief		WDT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Configure(void)
{

	if(HAL_WDT_Init() != HAL_OK)  
	{
		/* Initialization Error */
    		Error_Handler();
	}
	HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_LSI, 1);
	
	wdtCfg.wdtResetEn = ENABLE;
	wdtCfg.wdtCountEn = DISABLE;
	wdtCfg.wdtClkSel = SET; //SET:external from MCCR3, RESET:PCLK
	wdtCfg.wdtPrescaler = WDT_DIV_1;  //WDTCLK = WDTCLKIN / PRS =  40khz / 1
	wdtCfg.wdtTmrConst = 40000; //1s
	
	HAL_WDT_Configure(wdtCfg);

}
/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt

}
	

/**********************************************************************
 * @brief		WDT_Start
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Start(void)
{
	HAL_WDT_Start(ENABLE);
	i=20;j=0;
}

/**********************************************************************
 * @brief		WDT_ResetRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_ResetRun(void)
{
	msec = 500;
		while(msec);

		HAL_WDT_ReloadTimeCounter(40000);
		i--;
		
		if (j==0){
			j=1;
			HAL_GPIO_ClearPin(PB, _BIT(0));
		}
		else {
			j=0;
			HAL_GPIO_SetPin(PB, _BIT(0));
		}

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*GPIO Configure*/
	GPIO_Configure();
	
	/*Configure WDT */
	WDT_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();
  
	/*WDT Start*/
	WDT_Start();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(i)
	{
               /* Infinite loop */
		WDT_ResetRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
